(* analytic continuation of hypergeometric functions *)

BeginPackage["Hypergeometric`"];

HypergeometricPFQCont::usage =
   "HypergeometricPFQCont[{a1,...,ap},{b1,...,bq},z]" \
   <> " is the analytic continuation of the generalized hypergeometric function"

Begin["`Private`"];
Needs["HarmonicSums`"];
HypergeometricPFQCont::nocont = "No analytic continuation known for " <>
                                "HypergeometricPFQCont[`1`,`2`,`3`]";

HypergeometricPFQCont[{p___},{q___,n_},z_] := Module[
   { result, P = Length[{p}], Q = Length[{q}] + 1 },

   (* try Mellin transformed PolyLogs *)
   If[ (Q==P-1) && MatchQ[{p},{1..}] && MatchQ[{q},{2..}] && !PossibleZeroQ[n-1],
      Return[(n-1) * MellinPolyLog[P-2,n-2]]
   ];

   (* try built-in hypergeometric function *)
   result = HypergeometricPFQ[{p},{q,n},z];
   If[NumericQ[result], Return[result]];

   Message[HypergeometricPFQCont::nocont,{p},{q,n},z];
   Return[Hold[HypergeometricPFQCont[{p},{q,n},z]]];
] /; And @@ (NumericQ /@ {p,q,n,z});

MellinPolyLog[2,n_] := (
   - Zeta[2]*HarmonicSum[{1}, n] \
   + HarmonicSum[{1}, n]*HarmonicSum[{2}, n] \
   + HarmonicSum[{3}, n] \
   - HarmonicSum[{2, 1}, n] \
   + Zeta[3]
);

MellinPolyLog[3,n_] := (
   + 1/2*(
      + HarmonicSum[{1}, n]^2*PolyGamma[1, n + 1] \
      + Zeta[2]*HarmonicSum[{2}, n] \
      - HarmonicSum[{4}, n] \
   ) \
   + HarmonicSum[{3, 1}, n] \
   - HarmonicSum[{2, 1, 1}, n] \
   - HarmonicSum[{1}, n]*(
      + HarmonicSum[{3}, n] \
      - HarmonicSum[{2, 1}, n] \
      + Zeta[3]
   )
   + Zeta[4]
);

MellinPolyLog[4,n_] := (
   - HarmonicSum[{3}, n]/18*(Pi^2 - 15*HarmonicSum[{2}, n]) \
   + HarmonicSum[{5}, n] \
   + HarmonicSum[{2}, n]*HarmonicSum[{2, 1}, n]/2 \
   - HarmonicSum[{3, 2}, n] \
   + HarmonicSum[{1}, n]*(\
      - Zeta[4] \
      - Pi^2/12*HarmonicSum[{2}, n] \
      + HarmonicSum[{4}, n]/2 \
      - HarmonicSum[{3, 1}, n] \
      + HarmonicSum[{2, 1, 1}, n]
   ) \
   - HarmonicSum[{2, 2, 1}, n] \
   + HarmonicSum[{3, 1, 1}, n] \
   - HarmonicSum[{2, 1, 1, 1}, n] \
   - HarmonicSum[{1}, n]^3*PolyGamma[1, n + 1]/6 \
   + HarmonicSum[{2}, n]*Zeta[3]/2 \
   + HarmonicSum[{1}, n]^2/2*(
      + HarmonicSum[{3}, n] \
      - HarmonicSum[{2, 1}, n] \
      + Zeta[3]
   ) \
   + Zeta[5]
);

End[];
EndPackage[];
