(* ::Package:: *)

(************************************************************************)
(*                   QQbarGridCalc V2.0                                 *)
(************************************************************************)
(*
 v3 .6[15.11.2012]:
  1)decoration of authorship printing modified.
  2)duplication of Off[General:: ] and Off[NIntegrate:: ]
    in the last of this package deleted.
  3) the main body lapped by Begin["`Private`"] and End[].
  4) some functions are put into Unprotected[] and Protected[]
    in the Begin[] and End[].

 v3.1-v3.5[]: No update.

 v3.0 [28.06.2010] V3.0 created;

 2010.06.28: IntSmallVNA12[rh_,nn_] line break corrected;
 2010.06.28: IntSmallNANA2[rh_,nn_] line break corrected;
 2010.06.28: IntSmallVNA05[rh_,nn_] line break corrected;
 2010.05.30: Ins[\lambda,{1},{2},Sum1] for accuracy improvement;
 2010.05.30: Ins[\lambda,{1},{3},Sum1] for accuracy improvement;
 2010.05.30: Ins[\lambda,{1,1},{0,2},Sum1] for accuracy improvement;
 2010.05.30: Ins[\lambda,{1,1},{1,2},Sum1] for accuracy improvement;
 2010.05.30: Ins[\lambda,{1,1,1},{1,1,1},Sum1] for accuracy improvement;
 2010.05.30: Ins[\lambda,{1,1,1},{1,1,1},Sum2] for accuracy improvement;
 2010.05.30: Ins[\lambda,{1,1,1},{1,1,1},Sum3] for accuracy improvement;
 2010.05.30:Ins[\lambda,{1,1/2},{1,0},Sum2] for accuracy improvement;
*)

Print["Doing some precalculations..."]

Get["us.m"]; (* ultrasoft contribution *)
Get["non-resonant.m"];

Off[General::spell];
Off[General::spell1];
Off[SequenceLimit::seqlim];
Off[NIntegrate::slwcon];
Off[NIntegrate::ncvb];
Off[NIntegrate::ploss];
Off[NIntegrate::tmap];

$MaxExtraPrecision = 100;


BeginPackage["QQbarThreshold`"]

Needs["HarmonicSums`"]; (* analytic continuation for harmonic sums *)
Needs["Hypergeometric`"]; (* hypergeometric functions *)

QQbarCalcGrid::usage =
"QQbarCalcGrid[Energy -> {energies}, Width -> {widths}, gridfilename]"<>
" generates a grid for the cross section and saves the result"<>
" in a file with the given name. The argument energies should either be a List"<>
" of values for E = Sqrt[s] - 2*m or a Sequence of minimum value,"<>
" maximum value, and step width. Likewise width specifies the values of the"<>
" heavy quark width";

QuarkMass::usage =
"Option for QQbarCalcGrid. Specifies the mass of the heavy quark.";
alphaS::usage =
"Option for QQbarCalcGrid. Specifies the value of the strong coupling constant.";
Comments::usage =
"Option for QQbarCalcGrid and QQbarCalcNonresonantGrid."
" Specifies a list of comment lines to be prepended to the grid file.";
Options[QQbarCalcGrid] = {
   QuarkMass -> 175,
   alphaS -> 0.14,
   Comments -> Automatic
};

Energy;
Width;

QQbarCalcNonresonantGrid::usage =
"QQbarCalcNonresonantGrid[MassRatio -> {ratios}, Cut -> {ywcuts}, gridfilename]"<>
" generates a grid for the nonresonant cross section and saves the result"<>
" in a file with the given name. The argument ratios should either be a List"<>
" of values for x=mW^2/mtop^2 or a Sequence of minimum value, maximum value,"<>
" and step width. Likewise ywcuts specifies the values of yw = (1-y)/(1-x) "<>
" with y = (1 - DeltaM/mtop)^2, where 0 <= DeltaM <= mtop - mW.";

MassRatio;
Cut;

QQbarCalcNNLONonresonantGridEntry::usage =
"QQbarCalcNNLONonresonantGridEntry[x, yw] calculates a single entry"<>
" {automated, manual} in the grid for the NNLO nonresonant cross section."<>
" The coordinates are x = mW^2/mtop^2, yw = (1-y)/(1-x) with"<>
" y = (1 - DeltaM/mtop)^2, where 0 <= DeltaM <= mtop - mW."<>
" This function can only be used if the Docker image"<>
" amaier/nnlo_nonres_grid_entry is available."

Verbose::usage =
"Option for QQbarCalcNNLONonresonantGridEntry. Specifies whether intermediate"<>
" output should be printed";
PollFrequency::usage =
"Option for QQbarCalcNNLONonresonantGridEntry. Specifies the interval in"<>
" seconds between status updates";
DockerCommand::usage =
"Option for QQbarCalcNNLONonresonantGridEntry. Specifies the command to"<>
" start docker";


Begin["`Private`"];
Needs["ApproxSum`"];
Get["non-resonant.m"];
Unprotect[Ins,HypIns1Der,HypIns2Der,HypIns3Der];
QQbarGridCalcVersion = "1.0";

FilterOptions::usage = "f_[args___,FilterOptions[opts__]] returns f[args,fopts],"<>
" where fopts are those options from opts that are actually known to f."<>
" FilterOptions[g_, opts___] adds the default options assigned to the Symbol g.";

FilterOptions /: f_[args___, FilterOptions[g_Symbol, opts___]] :=
f[args, FilterOptions@@(
   {opts}~Join~Complement[Options[g], FilterRules[Options[g], {opts}]]
)];

FilterOptions /: f_[args___, FilterOptions[opts___]] :=
f[args, Sequence@@FilterRules[{opts}, Options[f]]];

Options[Ins] = Options[QQbarCalcGrid];
Options[MakeGrid] = Options[QQbarCalcGrid];

QQbarCalcGrid[
   Energy -> {LowerEnValue_, UpperEnValue_, EnStep_},
   Width -> {LowerWValue_, UpperWValue_, WStep_},
   gridfilename_String,opts:OptionsPattern[]
] := Module[
   {time0,timetest,starttime},
   (* TODO: reasonable time estimate; move to MakeGrid *)
   time0 = SessionTime[];
   AA[IIA, ex1][0 + 0.4*I, 10^9];
   timetest = (SessionTime[] - time0)*34;
   starttime = SessionTime[];
   Print["Calculation running... (Started at: ", Date[],
      "; Estimated finishing date: ",
      ToDate[FromDate[Date[]] +
   timetest*((UpperEnValue - LowerEnValue)/EnStep + 1)*((UpperWValue - LowerWValue)/WStep + 1)], ")"];
   Return[
      MakeGrid[
         {LowerEnValue, UpperEnValue, EnStep},
         {LowerWValue, UpperWValue, WStep},
         gridfilename,
         opts
      ]
   ];
];

QQbarCalcGrid[
   Energy -> {energies_List},
   Width -> {widths_List},
   gridfilename_String,
   opts:OptionsPattern[]
] := MakeGrid[
   {energies},
   {widths},
   gridfilename,
   opts
];

MakeGrid[
   energies_List,
   widths_List,
   gridfilename_String,
   opts:OptionsPattern[]
] :=
Module[
   {
      tab,starttime,
      comments = List /@ OptionValue[Comments]
   },
   starttime = SessionTime[];
   tab = GridTable[energies, widths, FilterOptions[opts]];
   If[comments === Automatic,
      comments = DefaultComments[energies, widths, gridfilename, opts]
   ];
   ExportGrid["tsv", tab, gridfilename, comments];

   Print["GRID FILE COMPLETED. Total time: ",
   Round[SessionTime[] - starttime], " seconds. Finished at: ", Date[]];
];

GridTable::usage = "Main function for calculating a grid";
Options[GridTable] = FilterRules[
   Options[MakeGrid],
   {QuarkMass, alphaS}
];

GridTable[
   {energies___},
   {widths___},
   opts:OptionsPattern[]
] := Module[
   {
      Lam, et, wt,
      cF = 4/3,
      as = OptionValue[alphaS],
      mt = OptionValue[QuarkMass]
   },
   Lam[et_,wt_]:=(-((4*(et + I*wt))/(cF^2*mt*as^2)))^(-2^(-1));
   Return[
      ParallelTable[{
         Lam[et,wt]^-2,
         Ins[Lam[et,wt], {1}, {2}, Sum1],
         Ins[Lam[et,wt], {1}, {3}, Sum1],
         Ins[Lam[et,wt], {1, 1}, {0, 1}, Sum1],
         Ins[Lam[et,wt], {1, 1}, {1, 1}, Sum1],
         Ins[Lam[et,wt], {1, 1}, {0, 2}, Sum1],
         Ins[Lam[et,wt], {1, 1}, {0, 2}, Sum2],
         Ins[Lam[et,wt], {1, 1}, {1, 2}, Sum1],
         Ins[Lam[et,wt], {1, 1}, {1, 2}, Sum2],
         Ins[Lam[et,wt], {1, 1}, {1, 2}, Sum3],
         Ins[Lam[et,wt], {1, 1, 1}, {0, 0, 1}, Sum1],
         Ins[Lam[et,wt], {1, 1, 1}, {0, 1, 0}, Sum1],
         Ins[Lam[et,wt], {1, 1, 1}, {1, 0, 1}, Sum1],
         Ins[Lam[et,wt], {1, 1, 1}, {0, 1, 1}, Sum1],
         Ins[Lam[et,wt], {1, 1, 1}, {0, 1, 1}, Sum2],
         Ins[Lam[et,wt], {1, 1, 1}, {1, 1, 1}, Sum1],
         Ins[Lam[et,wt], {1, 1, 1}, {1, 1, 1}, Sum2],
         Ins[Lam[et,wt], {1, 1, 1}, {1, 1, 1}, Sum3],
         Ins[Lam[et,wt], {1, 1/2}, {1, 0}, Sum1],
         Ins[Lam[et,wt], {1, 1/2}, {1, 0}, Sum2],
         Ins[Lam[et,wt], {1/2}, {1}, Sum1],
         Ins[Lam[et,wt], {1, del}, {1, 0}, Sum1],
         Ins[Lam[et,wt], {0}, {1}, Sum1],
         Ins[Lam[et,wt], {0}, {1}, Sum2],
         Ins[Lam[et,wt], US, FilterOptions[opts]],
         HypIns1Der[Lam[et,wt], 1],
         HypIns2Der[Lam[et,wt], 1],
         HypIns3Der[Lam[et,wt], 1]},
         {et, energies},
         {wt, widths}
         ,
         DistributedContexts -> All
      ]
   ]
];

DefaultComments::usage = "Default comments to be prepended to grid file";
DefaultComments[
   energies_,
   widths_,
   gridfilename_String,
   opts___
] := Module[
   {shorten = shorten[#, 4] &},
   {
      {"Generated by QQbarGridCalc "<>ToString[QQbarGridCalcVersion]},
      {"with"},
      {"QQbarCalcGrid["},
      {"   "<>ToString[Energy -> shorten /@ energies, InputForm]<>","},
      {"   "<>ToString[Width -> shorten /@ widths, InputForm]<>","},
      {"   "<>StringRiffle[ToString[#, InputForm] &/@ {gridfilename, opts}, ", "]},
      {"]"},
      {DateString[]}
   }
];

ExportGrid::usage = "Exports a grid to a file in the given output format";

ExportGrid["tsv", grid_, gridfilename_String, comments_] := Export[
   gridfilename,
   Join[Prepend[Rest[#], "# " <> First[#]] & /@ comments, Flatten[grid, 1]],
   "tsv"
];

shorten::usage = "shorten[l_, cut_] gives a shortened form of the List l "<>
" if l is longer than cut.";
shorten[l_List, cut_Integer] := If[
   Length[l] <= cut,
   l,
   With[
      {partLength = Floor[cut/2]},
      Join[Take[l, partLength], {"..."}, Take[l, -partLength]]
   ]
];
shorten[x_?NumberQ, _Integer] := x;

(***************************************************)
(* Nonresonant Grid                                *)
(***************************************************)

Options[QQbarCalcNonresonantGrid] := {
   Comments -> Automatic
};

QQbarCalcNonresonantGrid::ywrange =
"Cut parameter yw = `1` outside physical range:"<>
" 0 <= yw <= 1";

QQbarCalcNonresonantGrid[
   MassRatio -> {xmin_?NumberQ, xmax_?NumberQ, xstep_?NumberQ},
   Cut -> {ywmin_?NumberQ, ywmax_?NumberQ, ywstep_?NumberQ},
   filename_String,
   opts:OptionsPattern[]
] := (
   If[(# < 0) || (# > 1),
      Message[QQbarCalcNonresonantGrid::ywrange, #]
   ] & /@ {ywmin, ywmax};
   MakeNonresonantGrid[
      {xmin, xmax, xstep},
      {ywmin, ywmax, ywstep},
      filename,
      opts
   ]
);

QQbarCalcNonresonantGrid[
   MassRatio -> {xs_List},
   Cut -> {yws_List},
   filename_String,
   opts:OptionsPattern[]
] := (
   If[(# < 0) || (# > 1),
      Message[QQbarCalcNonresonantGrid::ywrange, #]
   ] & /@ yws;
   MakeNonresonantGrid[
      {xs},
      {yws},
      filename,
      opts
   ]
);

hids = {
   "1V", "1A", "1VA", "2V", "2A", "3V", "3A", "4V", "4A",
   "5", "6", "7", "8", "9", "10"
};

MakeNonresonantGrid::usage "Main function for generation of Nonresonant grid";
Options[MakeNonresonantGrid] = Options[QQbarCalcNonresonantGrid];
MakeNonresonantGrid[
   xs_List, yws_List, filename_String,
   opts:OptionsPattern[]
] := Module[
   {
      tab,starttime,
      comments = List /@ OptionValue[Comments]
   },
   starttime = SessionTime[];
   tab = NonresonantTable[xs, yws];
   If[comments === Automatic,
      comments = DefaultNonresonantComments[xs, yws, filename, opts];
      Print[InputForm[comments]];
   ];
   ExportGrid["tsv", tab, filename, comments];
   Print[
      "Nonresonant grid completed.",
      " Total time: ",Round[SessionTime[] - starttime], " seconds.",
      " Finished at: ", DateString[]
   ];
];

DefaultNonresonantComments::usage =
"Default comments to be prepended to nonresonant grid file";
DefaultNonresonantComments[
   xs_, yws_, filename_String,
   opts___
] := Module[
   {shorten = shorten[#, 4] &},
   {
      {"Generated by QQbarGridCalc "<>ToString[QQbarGridCalcVersion]},
      {"with"},
      {"QQbarCalcNonresonantGrid["},
      {"   "<>ToString[MassRatio -> shorten /@ xs, InputForm]<>","},
      {"   "<>ToString[Cut -> shorten /@ yws, InputForm]<>","},
      {"   "<>StringRiffle[ToString[#, InputForm] &/@ {filename, opts}, ", "]},
      {"]"},
      {DateString[]},
      {"Results and conventions taken from arXiv:1004.2188"},
      {"Coordinates are"},
      {"x = m_w^2/m_top^2"},
      {"yw = (1-y)/(1-x) with y = (1 - Delta_m/m_top)^2"},
      {""},
      Join[{"x","yw"}, hids]
   }
];

NonresonantTable[{xs___}, {yws___}] := Module[
   {ep = 10^-11, x, yw, id, tab, htab},
   (* compute with maximum parallelisation *)
   tab = Flatten[
      ParallelTable[
         {x, yw, id, Quiet[h[id, x,  1-ep-(1-ep-x*(1+ep))*yw]]},
         {x, xs},
         {yw, yws},
         {id, hids},
         DistributedContexts -> All
      ],
      2
   ];
   (* reformat *)
   htab = ({h, Most[#]} -> Last[#]) & /@ tab;
   Return[
      Table[
         Join[{x, yw}, {h, {x, yw, #}} &/@ hids],
         {x, xs},
         {yw, yws}
      ] /. htab
   ];
];

Options[QQbarCalcNNLONonresonantGridEntry] := {
   Verbose -> True,
   PollFrequency -> 1,
   DockerCommand -> "docker"
};

QQbarCalcNNLONonresonantGridEntry[
   x_?NumberQ, yw_?NumberQ, OptionsPattern[]
] := Module[
   {process, processout, lastout},
   process = StartProcess[{
      OptionValue[DockerCommand], "run", "amaier/nnlo_nonres_grid_entry",
      ToString[x], ToString[yw]
   }];
   If[process == $Failed, Return[$Failed]];
   While[ProcessStatus[process, "Running"],
      processout = ReadString[process, EndOfBuffer];
      If[processout =!= "",
         lastout = processout;
         If[OptionValue[Verbose], WriteString[$Output, lastout]];
      ];
      Pause[OptionValue[PollFrequency]];
   ];
   processout = ReadString[process, EndOfBuffer];
   If[processout =!= "", lastout = processout];
   If[OptionValue[Verbose], WriteString[$Output, lastout]];
   ToExpression /@ Take[StringSplit[lastout], -2]
];

(***************************************************)
(* Potential Calculation                           *)
(***************************************************)


Ins[lam_?NumberQ, {1}, {2}, Sum1] := (-2*lam*(
   - 2/lam^2 + EulerGamma/lam^2 + 6/lam - (2*EulerGamma)/lam + (2*Pi^2)/3 \
   - (EulerGamma*Pi^2)/6 - Pi^4/120 + HarmonicSum[{1},-1-lam]/lam^2 \
   - (2*HarmonicSum[{1},-1-lam])/lam - (Pi^2*HarmonicSum[{1},-1-lam])/2 \
   + (EulerGamma*Pi^2*HarmonicSum[{1},-1-lam])/6 \
   + (Pi^2*HarmonicSum[{1},-1-lam]^2)/6 - 4*HarmonicSum[{2},-1-lam] \
   + EulerGamma*HarmonicSum[{2},-1-lam] + (Pi^2*HarmonicSum[{2},-1-lam])/6 \
   + 3*HarmonicSum[{1},-1-lam]*HarmonicSum[{2},-1-lam] \
   - EulerGamma*HarmonicSum[{1},-1-lam]*HarmonicSum[{2},-1-lam] \
   - HarmonicSum[{1},-1-lam]^2*HarmonicSum[{2},-1-lam] \
   - HarmonicSum[{2},-1-lam]^2/2 + 2*HarmonicSum[{3},-1-lam] \
   - EulerGamma*HarmonicSum[{3},-1-lam] \
   - HarmonicSum[{1},-1-lam]*HarmonicSum[{3},-1-lam] \
   - HarmonicSum[{4},-1-lam]/2 \
   + (-2 + EulerGamma + HarmonicSum[{1},-1-lam])*HarmonicSum[{2,1},-1-lam] \
   + 2*Zeta[3] - EulerGamma*Zeta[3] - HarmonicSum[{1},-1-lam]*Zeta[3]
   ) \
   + 4*NSum[((n - lam)*PolyGamma[0, n - lam])/n^3, {n, 1, Infinity}]
);

(* ********** original formula (for reference) **********
 * Ins[lam_, {1}, {2}, Sum1] := (
 *    -2*Gamma[1 - lam]*Sum[
 *       (Gamma[-1 + k]*PolyGamma[0, -1 + k])/
 *       (k^2*Gamma[k - lam]),
 *       {k, 2, Infinity}
 *    ] +
 *    2*Sum[
 *       ((Gamma[-1 + k]*Gamma[1 - lam])*
 *       (-PolyGamma[0, -1 + k] + PolyGamma[0, k - lam]))/
 *       (k^2*Gamma[k - lam]),
 *       {k, 2, 500}
 *    ] +
 *    4*NSum[
 *       ((n - lam)*PolyGamma[0, n - lam])/n^3,
 *       {n, 1, Infinity}
 *    ]
 * );
 **********************************************************)

Ins[lam_, {1}, {3}, Sum1] := (
   + Ins[lam, {1}, {3}, kSum1] \
   + Ins[lam, {1}, {3}, kSum2] \
   + Ins[lam, {1}, {3}, hyper] \
);

Ins[lam_, {1}, {3}, kSum1] := -lam*(
   (-192*HarmonicSum[{1, 1}, -1 - lam])/lam^3 + (8*(-18 + lam*(36 +
   24*EulerGamma + 12*EulerGamma^2 + Pi^2) - 6*(-3 + (6 +
   4*EulerGamma)*lam)* HarmonicSum[{1}, -1 - lam] +
   18*lam*HarmonicSum[{1}, -1 - lam]^2)*HarmonicSum[{2, 1}, -1 - lam])/
   lam - (48*(1 + 4*(2 + EulerGamma)*lam - 8*lam*HarmonicSum[{1}, -1 -
   lam])* HarmonicSum[{3, 1}, -1 - lam])/lam + 240*HarmonicSum[{3, 2},
   -1 - lam] + 48*HarmonicSum[{4, 1}, -1 - lam] + (96*(-1 + 2*(1 +
   EulerGamma)*lam - 2*lam*HarmonicSum[{1}, -1 - lam])* HarmonicSum[{2,
   1, 1}, -1 - lam])/lam + 288*HarmonicSum[{2, 2, 1}, -1 - lam] -
   192*HarmonicSum[{3, 1, 1}, -1 - lam] + (2*(720 + 1440*EulerGamma*lam +
   720*EulerGamma^2*lam - 5760*lam^2 - 4320*EulerGamma*lam^2 -
   1440*EulerGamma^2*lam^2 + 60*lam*Pi^2 + 120*lam^2*Pi^2 -
   360*lam^3*Pi^2 - 240*EulerGamma*lam^3*Pi^2 -
   120*EulerGamma^2*lam^3*Pi^2 - 20*lam^2*Pi^4 + 5*lam^3*Pi^4 -
   16*EulerGamma*lam^3*Pi^4 + 60*lam^3*HarmonicSum[{1}, -1 - lam]^3*
   (Pi^2 - 6*HarmonicSum[{2}, -1 - lam]) + 180*lam^2*(3 +
   5*lam)*HarmonicSum[{2}, -1 - lam]^2 - 3600*lam^3*HarmonicSum[{3},
   -1 - lam] - 1440*EulerGamma*lam^3*HarmonicSum[{3}, -1 - lam] -
   720*EulerGamma^2*lam^3*HarmonicSum[{3}, -1 - lam] +
   300*lam^3*Pi^2*HarmonicSum[{3}, -1 - lam] +
   1260*lam^2*HarmonicSum[{4}, -1 - lam] + 1980*lam^3*HarmonicSum[{4},
   -1 - lam] + 720*EulerGamma*lam^3*HarmonicSum[{4}, -1 - lam] -
   1440*lam^3*HarmonicSum[{5}, -1 - lam] + 2160*lam^2*Zeta[3] -
   720*lam^3*Zeta[3] - 1440*EulerGamma*lam^3*Zeta[3] -
   720*EulerGamma^2*lam^3*Zeta[3] - 300*lam^3*Pi^2*Zeta[3] -
   60*HarmonicSum[{1}, -1 - lam]^2* (-12 + 6*lam - 12*lam^2 -
   2*lam^2*Pi^2 + 3*lam^3*Pi^2 + 2*EulerGamma*lam^3*Pi^2 - 6*lam^2*(-2 +
   (3 + 2*EulerGamma)*lam)*HarmonicSum[{2}, -1 - lam] +
   30*lam^3*HarmonicSum[{3}, -1 - lam] + 6*lam^3*Zeta[3]) -
   5*HarmonicSum[{1}, -1 - lam]* (-72 + 288*lam - 432*lam^2 +
   48*lam^2*Pi^2 - 72*lam^3*Pi^2 - 48*EulerGamma*lam^3*Pi^2 -
   24*EulerGamma^2*lam^3*Pi^2 + lam^3*Pi^4 + 72*lam^2*(-4 + lam*(6 +
   4*EulerGamma + 2*EulerGamma^2 - Pi^2))*HarmonicSum[{2}, -1 - lam] +
   180*lam^3*HarmonicSum[{2}, -1 - lam]^2 - 144*(5 +
   2*EulerGamma)*lam^3* HarmonicSum[{3}, -1 - lam] +
   396*lam^3*HarmonicSum[{4}, -1 - lam] + 432*lam^2*Zeta[3] -
   144*lam^3*Zeta[3] - 288*EulerGamma*lam^3*Zeta[3]) -
   120*HarmonicSum[{2}, -1 - lam]* (-6 - lam^2*(-12 + Pi^2) +
   15*lam^3*HarmonicSum[{3}, -1 - lam] + lam^3*(-6*EulerGamma^2 +
   EulerGamma*(-12 + Pi^2) + 3*(-6 + Pi^2 + 7*Zeta[3]))) +
   3420*lam^3*Zeta[5]))/(15*lam^3)
);

(* ********** original formula (for reference) **********
 * Ins[lam_, {1}, {3}, kSum1] := Sum[
 *    (
 *       Gamma[-1+k]*(
 *          (48*(4*PolyGamma[0,-1+k]-4*PolyGamma[0,k]-2*PolyGamma[0,1-lam]-2*PolyGamma[0,k-lam]))/k^3 +
 *          (48*(-PolyGamma[0,-1+k]^2-4*EulerGamma*PolyGamma[0,k]-2*PolyGamma[0,k]^2+PolyGamma[0,-1+k]*PolyGamma[0,1-lam]+PolyGamma[0,-1+k]*PolyGamma[0,k-lam]-PolyGamma[0,1-lam]*PolyGamma[0,k-lam]-PolyGamma[1,-1+k]+2*PolyGamma[1,k]))/k^2 +
 *          (48*(4*EulerGamma*PolyGamma[1,k]+4*PolyGamma[0,k]*PolyGamma[1,k]))/k
 *       )
 *    )/Gamma[k-lam],
 *    {k, 2, Infinity}
 * ];
 **********************************************************)

Ins[lam_, {1}, {3}, kSum2] = ApproxSum[
   (8*(-24 + 24*k - 12*EulerGamma*k - 12*k*lam + 12*EulerGamma*k*lam
   - 6*EulerGamma^2*k*lam + 12*k^2*lam - 12*EulerGamma*k^2*lam +
   6*EulerGamma^2*k^2*lam - k*lam*Pi^2 + k^2*lam*Pi^2 +
   6*k^2*PolyGamma[0, -1 + k]^2 + 6*k*(-2 - 3*(-2 + EulerGamma)*k +
   2*(-1 + EulerGamma)*k^2)* PolyGamma[0, k] + 6*(-1 +
   k)*k^2*PolyGamma[0, k]^2 + 6*k*PolyGamma[0, -1 + k]* (-4 +
   EulerGamma*k - k*PolyGamma[0, k]) + 24*k*PolyGamma[0, k - lam] -
   36*k^2*PolyGamma[0, k - lam] + 12*EulerGamma*k^2*PolyGamma[0, k -lam] +
   12*k^3*PolyGamma[0, k - lam] - 12*EulerGamma*k^3*PolyGamma[0, k - lam] -
   24*lam*PolyGamma[0, k - lam] + 36*k*lam*PolyGamma[0, k - lam] -
   12*EulerGamma*k*lam*PolyGamma[0, k - lam] -
   12*k^2*lam*PolyGamma[0, k - lam] +
   12*EulerGamma*k^2*lam*PolyGamma[0, k - lam] +
   6*k^2*PolyGamma[0, k- lam]^2 - 6*k^3*PolyGamma[0, k - lam]^2 -
   6*k*lam*PolyGamma[0, k - lam]^2 + 6*k^2*lam*PolyGamma[0, k - lam]^2 +
   24*lam*PolyGamma[0,-lam] - 24*k*lam*PolyGamma[0, -lam] +
   6*k^2*PolyGamma[1, -1 + k] + 6*k^2*PolyGamma[1, k] -
   6*k^3*PolyGamma[1, k] - 6*k^2*PolyGamma[1,k - lam] +
   6*k^3*PolyGamma[1, k - lam] + 6*k*lam*PolyGamma[1, k - lam] -
   6*k^2*lam*PolyGamma[1, k - lam]))/((-1 + k)*k^4),
   lam,
   {k,2,Infinity},
   ExpansionOrder -> 4,
   CutOff -> (600 &),
   AsymptoticPrecision -> 50
]

(* 2010.05.30 yk *)
 Ins[lam_, {1}, {3}, hyper] := (
   - 48*EulerGamma + 24*EulerGamma^2 - 8*EulerGamma^3 + 2*EulerGamma^4 - 36*Pi^2 -
   28*EulerGamma*Pi^2 + 6*EulerGamma^2*Pi^2 + (197*Pi^4)/30 -
   16*Pi^2*HypergeometricPFQCont[{1, 1, 1, 1}, {2, 2, 1 - lam}, 1] -
   192*(-1 + EulerGamma + (1 - lam)*PolyGamma[0, 1 - lam] + lam*PolyGamma[0, -lam]) -
   96*((1/12)*(12*(-1 + EulerGamma)*EulerGamma - 6*EulerGamma^2 + Pi^2 - 12*lam +
   12*EulerGamma*lam - 6*EulerGamma^2*lam - Pi^2*lam +
   12*(-1 + EulerGamma)*(1 - lam)*PolyGamma[0, 1 - lam] +
   6*(1 - lam)*PolyGamma[0, 1 - lam]^2 - 6*PolyGamma[1, 1 - lam] +
   6*lam*PolyGamma[1, 1 - lam])) + 2*PolyGamma[0, -lam]^4 +
   96*PolyGamma[1, 1 - lam] - 48*(-EulerGamma - PolyGamma[0, 1 - lam])*PolyGamma[1, 1 - lam] -
   16*(-6*EulerGamma^2 - Pi^2 - 12*EulerGamma*PolyGamma[0, 1 - lam] -
   6*PolyGamma[0, 1 - lam]^2 + 6*PolyGamma[1, 1 - lam]) +
   168*PolyGamma[1, -lam] - 168*EulerGamma*PolyGamma[1, -lam] +
   84*EulerGamma^2*PolyGamma[1, -lam] - 22*Pi^2*PolyGamma[1, -lam] + 48*lam*PolyGamma[1, -lam] -
   48*EulerGamma*lam*PolyGamma[1, -lam] +
   24*EulerGamma^2*lam*PolyGamma[1, -lam] -
   8*EulerGamma^3*lam*PolyGamma[1, -lam] +
   28*Pi^2*lam*PolyGamma[1, -lam] - 12*EulerGamma*Pi^2*lam*PolyGamma[1, -lam] + 6*PolyGamma[1, -lam]^2 +
   72*lam*PolyGamma[1, -lam]^2 - 72*EulerGamma*lam*PolyGamma[1, -lam]^2 +
   8*PolyGamma[0, -lam]^3*(-1 + EulerGamma - lam*PolyGamma[1, -lam]) -
   24*PolyGamma[2, 1 - lam] + 8*(2*EulerGamma^3 + EulerGamma*Pi^2 +
   6*EulerGamma*PolyGamma[0, 1 - lam]^2 + 2*PolyGamma[0, 1 - lam]^3 +
   PolyGamma[0, 1 - lam]*(6*EulerGamma^2 + Pi^2 - 6*PolyGamma[1, 1 - lam]) -
   6*EulerGamma*PolyGamma[1, 1 - lam] - 2*PolyGamma[2, 1] +
   2*PolyGamma[2, 1 - lam]) - 8*PolyGamma[2, -lam] +
   8*EulerGamma*PolyGamma[2, -lam] - 96*I*Pi*PolyGamma[2, -lam] -
   72*lam*PolyGamma[2, -lam] + 72*EulerGamma*lam*PolyGamma[2, -lam] -
   36*EulerGamma^2*lam*PolyGamma[2, -lam] + 14*Pi^2*lam*PolyGamma[2, -lam] +
   28*lam*PolyGamma[1, -lam]*PolyGamma[2, -lam] +
   6*PolyGamma[0, -lam]^2*(4 - 4*EulerGamma + 2*EulerGamma^2 + Pi^2 +
   (14 - 4*(-1 + EulerGamma)*lam)*PolyGamma[1, -lam] - 6*lam*PolyGamma[2, -lam]) -
   32*Pi^2*(Pi^2/3 - 2*PolyGamma[1, -lam] + lam*PolyGamma[2, -lam]) +
   46*PolyGamma[3, -lam] - 8*lam*PolyGamma[3, -lam] +
   8*EulerGamma*lam*PolyGamma[3, -lam] +
   32*I*Pi*lam*PolyGamma[3, -lam] - 14*lam*PolyGamma[4, -lam] -
   8*(-((4*Pi^4)/15) + 4*PolyGamma[3, -lam] - lam*PolyGamma[4, -lam]) -
   8*(-96 - 24*EulerGamma - 4*EulerGamma^3 - 2*EulerGamma*Pi^2 -
   24*PolyGamma[0, 1 - lam] -
   24*EulerGamma*(-3 + HypergeometricPFQCont[{1, 1, 1, 1}, {2, 2, 1 - lam}, 1] +
   HypergeometricPFQCont[{1, 1, 1, 1, 1}, {2, 2, 2, 1 - lam}, 1] - lam*PolyGamma[1, 1 - lam]) +
   (1/(lam*Gamma[-lam]))*(24*Gamma[1 - lam]*(-4 + HypergeometricPFQCont[{1, 1, 1, 1}, {2, 2, 1 - lam}, 1] +
   HypergeometricPFQCont[{1, 1, 1, 1, 1}, {2, 2, 2, 1 - lam}, 1] +
   HypergeometricPFQCont[{1, 1, 1, 1, 1, 1}, {2, 2, 2, 2, 1 - lam}, 1] -
   lam*PolyGamma[1, 1 - lam])) + 2*(4 + 2*EulerGamma^3 +
   EulerGamma*Pi^2)*(1 + lam*PolyGamma[1, 1 - lam]) + 4*PolyGamma[2, 1] +
   2*PolyGamma[1, 1 - lam]*(6*EulerGamma^2 + Pi^2 - 2*EulerGamma^3*lam -
   EulerGamma*Pi^2*lam + 2*lam*PolyGamma[2, 1]) -
   (-12*EulerGamma + 6*EulerGamma^2*lam + Pi^2*lam)*PolyGamma[2, 1 - lam] -
   (1/6)*(6*EulerGamma^2 + Pi^2)*(-12 + 6*EulerGamma^2 + Pi^2 +
   12*HypergeometricPFQCont[{1, 1, 1, 1}, {2, 2, 1 - lam}, 1] +
   (12 - 12*EulerGamma + 6*EulerGamma^2 + Pi^2)*lam*PolyGamma[0, 1 - lam] -
   12*lam*PolyGamma[0, -lam] + 12*EulerGamma*lam*PolyGamma[0, -lam] -
   6*EulerGamma^2*lam*PolyGamma[0, -lam] -
   Pi^2*lam*PolyGamma[0, -lam] - 12*PolyGamma[1, 1 - lam] - 12*lam*PolyGamma[1, 1 - lam] +
   12*EulerGamma*lam*PolyGamma[1, 1 - lam] +
   6*lam*PolyGamma[2, 1 - lam]) + 4*PolyGamma[3, 1 - lam] -
   4*EulerGamma*lam*PolyGamma[3, 1 - lam] +
   2*EulerGamma*(6*EulerGamma^2 + Pi^2 +
   (-12*EulerGamma + 6*EulerGamma^2*lam + Pi^2*lam)*PolyGamma[1, 1 - lam] +
   6*(-1 + EulerGamma*lam)*PolyGamma[2, 1 - lam] + 2*lam*PolyGamma[3, 1 - lam]) -
   lam*PolyGamma[4, 1 - lam] + 8*(1 + lam*PolyGamma[1, 1 - lam])*(-1 + Zeta[3])) -
   32*I*Pi*(-3*PolyGamma[2, -lam] + lam*PolyGamma[3, -lam] - 6*Zeta[3]) +
   192*(EulerGamma - Pi^2/6 - Pi^4/90 + HypergeometricPFQCont[{1, 1, 1, 1}, {2, 2, 1 - lam}, 1] +
   HypergeometricPFQCont[{1, 1, 1, 1, 1}, {2, 2, 2, 1 - lam}, 1] +
   HypergeometricPFQCont[{1, 1, 1, 1, 1, 1}, {2, 2, 2, 2, 1 - lam}, 1] +
   PolyGamma[0, -lam] - lam*PolyGamma[1, -lam] - Zeta[3]) - 16*Zeta[3] +
   112*EulerGamma*Zeta[3] - 192*I*Pi*Zeta[3] - 112*lam*PolyGamma[1, -lam]*Zeta[3] -
   (8*Gamma[1 - lam]*(-12 + 12*EulerGamma - 6*EulerGamma^2 - Pi^2 +
   12*PolyGamma[0, -lam] - 12*EulerGamma*PolyGamma[0, -lam] -
   6*PolyGamma[0, -lam]^2 + 6*PolyGamma[1, -lam])*Zeta[3])/Gamma[-lam] +
   4*PolyGamma[0, -lam]*(-12 + 12*EulerGamma - 6*EulerGamma^2 +
   2*EulerGamma^3 - 7*Pi^2 + 3*EulerGamma*Pi^2 - 3*(14 +
   2*EulerGamma^2*lam + (4 + Pi^2)*lam - 2*EulerGamma*(7 + 2*lam))*PolyGamma[1, -lam] -
   18*lam*PolyGamma[1, -lam]^2 + (2 - 18*(-1 + EulerGamma)*lam)*PolyGamma[2, -lam] +
   2*lam*PolyGamma[3, -lam] + 28*Zeta[3])
);

Ins[lam_, {1, 1}, {0, 1}, Sum1] := (
   + lam*Zeta[2]^2 - HarmonicSum[{2, 1}, -lam] \
   - 2*lam*HarmonicSum[{3, 1}, -lam] + PolyGamma[2, 1 - lam]/2 \
   + lam/2*HarmonicSum[{1}, -lam]*PolyGamma[2, 1 - lam] \
   - lam/6*PolyGamma[3, 1 - lam] + 2*Zeta[3]
);

(* ********** original formula (for reference) **********
 * Ins[lam_, {1, 1}, {0, 1}, Sum1] := (
 *    Sum[
 *       ((1 + n + lam)*PolyGamma[0, 1 + n - lam])/(1 + n - lam)^3,
 *       {n, 0, Infinity}
 *    ] +
 *    (1/6)*(
 *       6*EulerGamma*PolyGamma[1, 1 - lam] -
 *       lam*(3*(EulerGamma - PolyGamma[0, 1 - lam])*PolyGamma[2, 1 - lam] - PolyGamma[3, 1 - lam])
 *    )
 * );
 **********************************************************)

Ins[lam_?NumberQ, {1, 1}, {1, 1}, Sum1] :=
NSum[(2*(1 + n + lam)*(EulerGamma*(1 + n)*(1 + n - lam) +
lam + lam*(-1 - n + lam)*PolyGamma[0, 1 - lam])*PolyGamma[0, 1 + n - lam])/((1 + n)*(1 + n - lam)^4), {n, 0, Infinity}] +
NSum[((1 + n + lam)^2*PolyGamma[0, 1 + n - lam]^2)/((1 + n)*(1 + n - lam)^3), {n, 0, Infinity}] -
PolyGamma[0, -lam]^3/lam + PolyGamma[0, -lam]^2*(3/lam^2 - PolyGamma[1, -lam] - (1/2)*lam*PolyGamma[2, -lam]) +
EulerGamma^2*(PolyGamma[1, -lam] - (1/2)*lam*PolyGamma[2, -lam]) + (1/6)*PolyGamma[3, -lam] +
EulerGamma*(-(PolyGamma[0, -lam]^2/lam) - PolyGamma[2, -lam] + PolyGamma[0, -lam]*(-(2/lam^2) +
lam*PolyGamma[2, -lam]) + (1/3)*lam*PolyGamma[3, -lam]) + PolyGamma[0, -lam]*(2/lam^3 -
(1/3)*lam*PolyGamma[3, -lam]) - (1/24)*lam*PolyGamma[4, -lam]


(* 2010.05.30 yk *)
(* note_Ins2 _R1 _R1.nb; a part of NumSum08 is Ins[le,{1,1},{0,2},Sum1] *)
Ins[lam_,{1,1},{0,2},Sum1] = ApproxSum[
   (k*lam*HarmonicSum[{1},k-1]*HarmonicSum[{1},-lam])/(k - lam)^3 +
   ((-2*PolyGamma[1, 1 + k - lam] + lam*PolyGamma[2, 1 + k - lam])*
   lam*HarmonicSum[{1},k-lam])/(2*k),
  lam,
  {k, 1, Infinity},
  ExpansionOrder -> 10,
  CutOff -> (Max[100,20*Abs[#]] &),
  AsymptoticPrecision -> 50
];

Ins[lam_, {1, 1}, {0, 2}, Sum2] := (
   + 7*Pi^4/360 + HarmonicSum[{2, 2}, -lam] \
   + 2*HarmonicSum[{3, 1}, -lam] + 2*lam*HarmonicSum[{3, 2}, -lam] \
   + 2*lam*HarmonicSum[{4, 1}, -lam] - 2*HarmonicSum[{2, 1, 1}, -lam] \
   - 4*lam*HarmonicSum[{3, 1, 1}, -lam] \
   + 5*Pi^2*PolyGamma[1, 1 - lam]/24 - PolyGamma[1, 1 - lam]^2/2 \
   - 5*lam*Pi^2*PolyGamma[2, 1 - lam]/48 + lam*Zeta[5] \
   + lam*PolyGamma[1, 1 - lam]*PolyGamma[2, 1 - lam]/4 \
   + PolyGamma[3, 1 - lam]/6 - lam*PolyGamma[4, 1 - lam]/24 \
   + HarmonicSum[{1}, -lam]^2*(
      + PolyGamma[1, 1 - lam]/2 \
      - lam*PolyGamma[2, 1 - lam]/4
   ) \
   + HarmonicSum[{1}, -lam]*(
      - lam*Pi^4/36 + HarmonicSum[{2, 1}, -lam] \
      + 2*lam*HarmonicSum[{3, 1}, -lam] - PolyGamma[2, 1 - lam]/2 \
      + lam*PolyGamma[3, 1 - lam]/6 - 2*Zeta[3]
   )
);

(* ********** original formula (for reference) **********
 * Ins[lam_, {1, 1}, {0, 2}, Sum2] := (
 *    NSum[((1 + n + lam)*PolyGamma[0, 1 + n - lam]^2)/(1 + n - lam)^3, {n, 0, Infinity}] +
 *    NSum[
 *       ((2*lam + EulerGamma*(1 + 2*n + n^2 - lam^2) - (1 + 2*n + n^2 - lam^2)*PolyGamma[0, 1 - lam])*PolyGamma[0, 1 + n - lam])/(1 + n - lam)^4,
 *       {n, 0, Infinity}
 *    ] +
 *    (1/48)*(
 *       -24*PolyGamma[1, 1 - lam]^2 +
 *       2*PolyGamma[1, 1 - lam]*(12*EulerGamma^2 + 5*Pi^2 + 12*PolyGamma[0, 1 - lam]^2 + 6*lam*PolyGamma[2, 1 - lam]) -
 *       lam*(
 *          (12*EulerGamma^2 + 5*Pi^2 - 24*EulerGamma*PolyGamma[0, 1 - lam] + 12*PolyGamma[0, 1 - lam]^2)*PolyGamma[2, 1 - lam] -
 *          8*(EulerGamma - PolyGamma[0, 1 - lam])*PolyGamma[3, 1 - lam] +
 *          2*PolyGamma[4, 1 - lam]
 *       )
 *    )
 * );
 **********************************************************)

Ins[lam_,{1, 1}, {1, 2},Sum1] := (
   + Ins[lam,{1, 1}, {1, 2},ApproxSum1] \
   + Ins[lam,{1, 1}, {1, 2},DoubleSum1]
);

Ins[lam_,{1, 1}, {1, 2},ApproxSum1] = ApproxSum[
   (lam*((HarmonicSum[{1},-lam]*(6*(-k^2 + lam^2)*HarmonicSum[{1},-1 + k - lam] +
   6*lam*(-1 + (k - lam)*HarmonicSum[{1},-lam]))*HarmonicSum[{1},k - 1])/
   (k - lam)^4 + (3*lam*HarmonicSum[{1},k - lam]*HarmonicSum[{1},-lam]*
   PolyGamma[2, 1 + k - lam])/k + (lam*HarmonicSum[{1},k - lam]*
   PolyGamma[3, 1 + k - lam])/k))/6,
   lam,
   {k,1,Infinity},
   ExpansionOrder -> 10,
   CutOff -> (Max[2000,10*Abs[#]] &),
   AsymptoticPrecision -> 100
];

Module[
   {SumExact,SumRest},
   Ins[lam_?NumberQ,{1, 1}, {1, 2},DoubleSum1] := Module[
      {CutOff = 4000, AsymptoticPrecision = 50},
      Return[
         + SumExact[lam,{1,CutOff}] \
         + SumRest[SetPrecision[lam,AsymptoticPrecision],{CutOff+1,Infinity}]
      ];
   ];

   SumExact[lam_,{k0_,kcut_}] := lam*Sum[
      HarmonicSum[{1},k-lam]/k*(
         Zeta[3]+lam/2*Zeta[4]-HarmonicSum[{2,1},k-lam]+HarmonicSum[{3},k-lam] \
         -2*lam*(HarmonicSum[{3,1},k-lam]-HarmonicSum[{4},k-lam])
      ),
      {k, k0, kcut}
   ];

   (* ********** original formula (for reference) **********
    * SumExact[lam_] := Sum[
    *    ((lam (i + lam) (EulerGamma + PolyGamma[0, i - lam]))/(i - lam)^3)*
    *    (EulerGamma + PolyGamma[0, 1 + k - lam])/k!,
    *    {i, 1 + k, Infinity},
    *    {k, 1, Infinity}
    * ];
    **********************************************************)


   (* sum over summand expanded up to k^-8 *)
   (* TODO: generate when package is loaded *)
   SumRest[lam_,{k0_,Infinity}] := EulerGamma*(1+EulerGamma)*lam*Zeta[2,k0]+(lam*(2+EulerGamma-4*lam-2*EulerGamma*lam+EulerGamma^2*(-2+8*lam))*Zeta[3,k0])/4+lam*(-5/24+lam*(1+lam)+EulerGamma*(-7/36+EulerGamma/6+lam*(5/3-3*lam+EulerGamma*(-2+3*lam))))*Zeta[4,k0]+(lam*(13-92*lam-6*lam^2+852*lam^3+72*EulerGamma^2*lam*(2-9*lam+8*lam^2)+EulerGamma*(6-174*lam+828*lam^2-888*lam^3))*Zeta[5,k0])/144+lam*(-97/2160-(EulerGamma*(-2+15*EulerGamma))/450+lam*(71/144+EulerGamma/4+lam*(-11/18+(lam*(-34+81*lam))/6+EulerGamma*(-9/2+3*EulerGamma+lam*(40/3-(59*lam)/6+EulerGamma*(-8+5*lam))))))*Zeta[6,k0]+lam*(71/7200+EulerGamma/60+lam*(-8081/21600+(43/360-EulerGamma/3)*EulerGamma+lam*(3/2+(5*EulerGamma)/6+lam*(937/432+(lam*(-13525+16926*lam))/720+EulerGamma*(-431/36+(20*EulerGamma)/3+lam*(25-(139*lam)/10+EulerGamma*(-25/2+6*lam)))))))*Zeta[7,k0]+lam*(571/37800+(EulerGamma*(-99+140*EulerGamma))/5880+lam*(37/400+EulerGamma/6+lam*(-3059/1800+(19/20-(3*EulerGamma)/2)*EulerGamma+lam*(55/16+(25*EulerGamma)/12+lam*(1661/144+(lam*(-2523+2144*lam))/60+EulerGamma*(-155/6+(25*EulerGamma)/2+lam*(206/5-(183*lam)/10+EulerGamma*(-18+7*lam))))))))*Zeta[8,k0]-(1+2*EulerGamma)*lam*Derivative[1,0][Zeta][2,k0]+lam*(-1/4+EulerGamma+lam/2-4*EulerGamma*lam)*Derivative[1,0][Zeta][3,k0]+lam*(7/36-EulerGamma/3+lam*(-5/3+EulerGamma*(4-6*lam)+3*lam))*Derivative[1,0][Zeta][4,k0]+lam*(-1/24+lam*(29/24-2*EulerGamma+lam*(-23/4+EulerGamma*(9-8*lam)+(37*lam)/6)))*Derivative[1,0][Zeta][5,k0]+lam*(-1/225+EulerGamma/15+lam*(-1/4+lam*(9/2-6*EulerGamma-(lam*(80-59*lam+12*EulerGamma*(-8+5*lam)))/6)))*Derivative[1,0][Zeta][6,k0]+lam*(-1/60+lam*(-43/360+(2*EulerGamma)/3+lam*(-5/6+lam*(431/36-(40*EulerGamma)/3+lam*(-25+EulerGamma*(25-12*lam)+(139*lam)/10)))))*Derivative[1,0][Zeta][7,k0]+lam*(33/1960-EulerGamma/21+lam*(-1/6+lam*(-19/20+3*EulerGamma+lam*(-25/12+lam*(155/6-25*EulerGamma+lam*(-206/5+EulerGamma*(36-14*lam)+(183*lam)/10))))))*Derivative[1,0][Zeta][8,k0]+lam*Derivative[2,0][Zeta][2,k0]+lam*(-1/2+2*lam)*Derivative[2,0][Zeta][3,k0]+lam*(1/6+lam*(-2+3*lam))*Derivative[2,0][Zeta][4,k0]+lam^2*(1+lam*(-9/2+4*lam))*Derivative[2,0][Zeta][5,k0]+lam*(-1/30+lam^2*(3+lam*(-8+5*lam)))*Derivative[2,0][Zeta][6,k0]+(lam^2*(-2+40*lam^2-75*lam^3+36*lam^4)*Derivative[2,0][Zeta][7,k0])/6+lam*(1/42+lam^2*(-3/2+lam^2*(25/2+lam*(-18+7*lam))))*Derivative[2,0][Zeta][8,k0];

];

Ins[lam_?NumberQ, {1, 1}, {1, 2}, Sum2] := NSum[(1/((1 + n)*(1 + n - \
lam)^4))*(PolyGamma[0, 1 + n - lam]*(3*EulerGamma + \
9*EulerGamma*n + 9*EulerGamma*n^2 + 3*EulerGamma*n^3 + 4*lam + 4*n*\
lam + 2*lam^2 - 3*EulerGamma*lam^2 - 3*EulerGamma*n*\
lam^2 -
       (1 + n^3 + 2*lam - lam^2 - 2*lam^3 + n^2*(3 + 2*\
lam) + n*(3 + 4*lam - lam^2))*PolyGamma[0, 1 - lam] + \
(2 + 2*n^3 + lam - 2*lam^2 - lam^3 + n^2*(6 + lam) + \
n*(6 + 2*lam - 2*lam^2))*PolyGamma[0, 1 + n - lam])), {n, \
0, Infinity}] -
   (1/(48*lam^3))*(48*EulerGamma + 48*lam^2*PolyGamma[0, 1 - \
lam]^3 + 48*lam*PolyGamma[1, 1 - lam] - 72*EulerGamma^2*\
lam^3*PolyGamma[1, 1 - lam] - 10*Pi^2*lam^3*PolyGamma[1, 1 \
- lam] + 24*lam^3*PolyGamma[1, 1 - lam]^2 + \
24*lam^2*PolyGamma[2, 1 - lam] +
     36*EulerGamma^2*lam^4*PolyGamma[2, 1 - lam] + 5*Pi^2*\
lam^4*PolyGamma[2, 1 - lam] - 12*lam^4*PolyGamma[1, 1 - \
lam]*PolyGamma[2, 1 - lam] +
     12*lam*PolyGamma[0, 1 - lam]^2*(8 + 4*EulerGamma*lam + \
2*lam^2*PolyGamma[1, 1 - lam] + 3*lam^3*PolyGamma[2, 1 - \
lam]) + 8*lam^3*PolyGamma[3, 1 - lam] - 24*EulerGamma*\
lam^4*PolyGamma[3, 1 - lam] +
     24*PolyGamma[0, 1 - lam]*(2 + 4*EulerGamma*lam + \
4*lam^2*PolyGamma[1, 1 - lam] + lam^3*(2 - 3*EulerGamma*\
lam)*PolyGamma[2, 1 - lam] + lam^4*PolyGamma[3, 1 - \
lam]) + 4*lam^4*PolyGamma[4, 1 - lam])


Ins[lam_?NumberQ, {1, 1}, {1, 2}, Sum3] := NSum[24*lam*(((1 + n + \
lam)*PolyGamma[0, 1 + n - lam]^2)/((1 + n)*(1 + n - \
lam)^4)) + 8*EulerGamma*(((1 + n + lam)*(2 + 2*n + \
lam)*PolyGamma[0, 1 + n - lam]^2)/((1 + n)*(1 + n - \
lam)^3)) +
     8*(((1 + n + lam)^2*PolyGamma[0, 1 + n - lam]^3)/((1 + n)*(1 \
+ n - lam)^3)) + (1/(3*(1 + n)*(1 + n - lam)^5))*(PolyGamma[0, 1 \
+ n - lam]*(36*EulerGamma^2 + 144*EulerGamma^2*n + 216*EulerGamma^2*n^2 \
+ 144*EulerGamma^2*n^3 + 36*EulerGamma^2*n^4 +
        5*Pi^2 + 20*n*Pi^2 + 30*n^2*Pi^2 + 20*n^3*Pi^2 + 5*n^4*Pi^2 + 24*\
lam + 96*EulerGamma*lam - 36*EulerGamma^2*lam + 24*n*\
lam + 192*EulerGamma*n*lam - 108*EulerGamma^2*n*lam + \
96*EulerGamma*n^2*lam - 108*EulerGamma^2*n^2*lam -
        36*EulerGamma^2*n^3*lam - 5*Pi^2*lam - \
15*n*Pi^2*lam - 15*n^2*Pi^2*lam - 5*n^3*Pi^2*lam + 72*\
lam^2 - 48*EulerGamma*lam^2 - 36*EulerGamma^2*lam^2 - \
48*EulerGamma*n*lam^2 - 72*EulerGamma^2*n*lam^2 - \
36*EulerGamma^2*n^2*lam^2 - 5*Pi^2*lam^2 -
        10*n*Pi^2*lam^2 - 5*n^2*Pi^2*lam^2 - 48*EulerGamma*\
lam^3 + 36*EulerGamma^2*lam^3 + 36*EulerGamma^2*n*lam^3 + \
5*Pi^2*lam^3 + 5*n*Pi^2*lam^3 - 48*lam*(1 + n^2 + lam \
- 2*lam^2 + n*(2 + lam))*PolyGamma[0, 1 - lam] -
        24*EulerGamma*(1 + 2*n + n^2 - lam^2)*(1 + n^2 + lam - 2*\
lam^2 + n*(2 + lam))*PolyGamma[0, 1 - lam] + 12*(1 + n - \
lam)^2*(1 + 2*n + n^2 + 3*lam + 3*n*lam + \
2*lam^2)*PolyGamma[0, 1 - lam]^2 -
        12*(1 + n)*(1 + n - lam)^2*(1 + n + lam)*PolyGamma[1, 1 - \
lam])) - 8*(((1 + n + lam)*(1 + n + 2*lam)*PolyGamma[0, 1 + \
n - lam]^2)/((1 + n)*(1 + n - lam)^3))*PolyGamma[0, 1 - \
lam], {n, 0, Infinity}] -
   (1/(90*lam^4))*(-720*EulerGamma + 720*EulerGamma^2*lam - 720*\
lam*PolyGamma[1, 1 - lam] + \
720*EulerGamma*lam^2*PolyGamma[1, 1 - lam] - 360*EulerGamma^3*\
lam^4*PolyGamma[1, 1 - lam] - \
150*EulerGamma*Pi^2*lam^4*PolyGamma[1, 1 - lam] +
     360*EulerGamma*lam^4*PolyGamma[1, 1 - lam]^2 - \
360*lam^2*PolyGamma[2, 1 - lam] + \
360*EulerGamma*lam^3*PolyGamma[2, 1 - lam] + 180*EulerGamma^3*\
lam^5*PolyGamma[2, 1 - lam] + \
75*EulerGamma*Pi^2*lam^5*PolyGamma[2, 1 - lam] -
     180*EulerGamma*lam^5*PolyGamma[1, 1 - lam]*PolyGamma[2, 1 - \
lam] - 180*lam^2*PolyGamma[0, 1 - lam]^3*(4 - 4*EulerGamma*\
lam + lam^3*PolyGamma[2, 1 - lam]) - \
120*lam^3*PolyGamma[3, 1 - lam] + \
120*EulerGamma*lam^4*PolyGamma[3, 1 - lam] -
     180*EulerGamma^2*lam^5*PolyGamma[3, 1 - lam] - 25*Pi^2*\
lam^5*PolyGamma[3, 1 - lam] + 60*lam^5*PolyGamma[1, 1 - \
lam]*PolyGamma[3, 1 - lam] +
     180*lam*PolyGamma[0, 1 - lam]^2*(-8 + 4*EulerGamma*lam \
+ 4*EulerGamma^2*lam^2 + 2*lam^2*(-2 + \
EulerGamma*lam)*PolyGamma[1, 1 - lam] + lam^3*(-2 + \
3*EulerGamma*lam)*PolyGamma[2, 1 - lam] - \
lam^4*PolyGamma[3, 1 - lam]) -
     30*lam^4*PolyGamma[4, 1 - lam] + \
60*EulerGamma*lam^5*PolyGamma[4, 1 - lam] - 15*PolyGamma[0, 1 - \
lam]*(lam^3*(48 - 48*EulerGamma*lam + 36*EulerGamma^2*\
lam^2 + 5*Pi^2*lam^2)*PolyGamma[2, 1 - lam] -
       12*lam^2*PolyGamma[1, 1 - lam]*(-8 + \
8*EulerGamma*lam + lam^3*PolyGamma[2, 1 - lam]) + 4*(12 + \
12*EulerGamma*lam - 24*EulerGamma^2*lam^2 + 2*lam^4*(2 - \
3*EulerGamma*lam)*PolyGamma[3, 1 - lam] + \
lam^5*PolyGamma[4, 1 - lam])) -
     6*lam^5*PolyGamma[5, 1 - lam])


Ins[lam_, {1, 1, 1}, {0, 0, 1}, Sum1] := (
   + 1/2*Zeta[2]^2 - 2*Zeta[2]*Zeta[3]*lam + 6*Zeta[5]*lam \
   - 1/6*PolyGamma[3, 1 - lam] \
   - 1/6*PolyGamma[3, 1 - lam]*HarmonicSum[{1}, -lam]*lam \
   + 1/24*PolyGamma[4, 1 - lam]*lam - HarmonicSum[{3, 1}, -lam] \
   - 2*HarmonicSum[{4, 1}, -lam]*lam
);

(* ********** original formula (for reference) **********
 * Ins[lam_, {1, 1, 1}, {0, 0, 1}, Sum1] := (
 *    NSum[
 *       ((1 + n + lam)*PolyGamma[0, 1 + n - lam])/(1 + n - lam)^4,
 *       {n, 0, Infinity}
 *    ] +
 *    (1/24)*(
 *       -12*EulerGamma*PolyGamma[2, 1 - lam] -
 *       lam*(-4*(EulerGamma - PolyGamma[0, 1 - lam])*PolyGamma[3, 1 - lam] + PolyGamma[4, 1 - lam])
 *    )
 * )
 **********************************************************)

Ins[lam_?NumberQ, {1, 1, 1}, {1, 0, 1}, Sum1] := NSum[((1 + n + \
lam)*(2*(EulerGamma*(1 + n)*(1 + n - lam) + lam) + 2*\
lam*(-1 - n + lam)*PolyGamma[0, 1 - lam])*PolyGamma[0, 1 + \
n - lam])/((1 + n)*(1 + n - lam)^5) +
     ((1 + n + lam)^2*PolyGamma[0, 1 + n - lam]^2)/((1 + n)*(1 + \
n - lam)^4), {n, 0, Infinity}] + (1/(120*lam^4))*(120*EulerGamma \
+ 120*lam^2*PolyGamma[0, 1 - lam]^3 + 120*lam*PolyGamma[1, \
1 - lam] + 60*lam^2*PolyGamma[2, 1 - lam] -
     60*EulerGamma^2*lam^4*PolyGamma[2, 1 - lam] + \
20*lam^3*PolyGamma[3, 1 - lam] + \
20*EulerGamma^2*lam^5*PolyGamma[3, 1 - lam] + \
20*lam*PolyGamma[0, 1 - lam]^2*(12 + 6*EulerGamma*lam + 6*\
lam^2*PolyGamma[1, 1 - lam] + 3*lam^3*PolyGamma[2, 1 - \
lam] +
       lam^4*PolyGamma[3, 1 - lam]) + 5*lam^4*PolyGamma[4, \
1 - lam] - 10*EulerGamma*lam^5*PolyGamma[4, 1 - lam] + \
10*PolyGamma[0, 1 - lam]*(12 + 24*EulerGamma*lam + \
24*lam^2*PolyGamma[1, 1 - lam] + 12*lam^3*PolyGamma[2, 1 - \
lam] +
       4*lam^4*PolyGamma[3, 1 - lam] - \
4*EulerGamma*lam^5*PolyGamma[3, 1 - lam] + \
lam^5*PolyGamma[4, 1 - lam]) + lam^5*PolyGamma[5, 1 - \
lam])


Ins[lam_, {1, 1, 1}, {0, 1, 0}, Sum1] := (
   - EulerGamma*PolyGamma[2, 1 - lam] \
   + 1/3*EulerGamma*PolyGamma[3, 1 - lam]*lam - 17/10*Zeta[2]^2 \
   - 4*Zeta[2]*Zeta[3]*lam + Zeta[2]*PolyGamma[1, 1 - lam] \
   - Zeta[2]*PolyGamma[2, 1 - lam]*lam + 3*Zeta[5]*lam \
   + 1/2*PolyGamma[3, 1 - lam] - 1/6*PolyGamma[4, 1 - lam]*lam \
   + HarmonicSum[{2, 2}, -lam] + 2*HarmonicSum[{3, 1}, -lam] \
   + 2*HarmonicSum[{3, 2}, -lam]*lam + 2*HarmonicSum[{4, 1}, -lam]*lam
);

(* ********** original formula (for reference) **********
 * Ins[lam_, {1, 1, 1}, {0, 1, 0}, Sum1] := NSum[
 *     (-2*(1 + n)*PolyGamma[0, 1 + n - lam] + (1 + 2*n + n^2 - lam^2)*PolyGamma[1, 1 + n - lam])/
 *     (1 + n - lam)^4,
 *     {n, 0, Infinity}
 * ]
 **********************************************************)

Module[
   {sum4},

   Ins[lam_?NumberQ, {1, 1, 1}, {0, 1, 1}, Sum1] := (
      - 2*lam^2*HarmonicSum[{2}, -lam]*HarmonicSum[{3}, -lam] \
      - lam^3*HarmonicSum[{3}, -lam]^2 - 2*lam^2*HarmonicSum[{5}, -lam] \
      - lam^3*HarmonicSum[{6}, -lam] \
      + lam*(2 - 4*EulerGamma*lam)*HarmonicSum[{2, 2}, -lam] \
      + 8*EulerGamma*lam^2*HarmonicSum[{3, 1}, -lam] \
      + lam^2*(6 - 8*EulerGamma*lam)*HarmonicSum[{3, 2}, -lam] \
      + 8*lam^2*(1 + 2*EulerGamma*lam)*HarmonicSum[{4, 1}, -lam] \
      + 4*lam^3*HarmonicSum[{4, 2}, -lam] + 16*lam^3*HarmonicSum[{5, 1}, -lam] \
      - 8*lam^2*HarmonicSum[{3, 1, 1}, -lam] \
      - 16*lam^3*HarmonicSum[{4, 1, 1}, -lam] \
      + lam*PolyGamma[3, 1 - lam]*(
         1/3 + lam*(EulerGamma + 5*EulerGamma^2*lam/3 - lam*Pi^2/9)
      ) \
      + lam^3*PolyGamma[0, 1 - lam]^2*PolyGamma[3, 1 - lam]/3 \
      - lam^2*(1 + 7*EulerGamma*lam)*PolyGamma[4, 1 - lam]/12 \
      + PolyGamma[0, 1 - lam]*(
         + EulerGamma*lam^2*PolyGamma[2, 1 - lam] \
         + lam^3*PolyGamma[1, 1 - lam]*PolyGamma[2, 1 - lam] \
         - 2*EulerGamma*lam^3*PolyGamma[3, 1 - lam]/3 \
         + lam^3*PolyGamma[4, 1 - lam]/6
      ) \
      + lam^3*PolyGamma[5, 1 - lam]/10 \
      + lam^2*PolyGamma[2, 1 - lam]*(
         - 3*EulerGamma^2 + 2*EulerGamma*lam*Pi^2/3 + lam*Zeta[3]
      ) \
      + PolyGamma[1, 1 - lam]*(
         - EulerGamma*lam^3*PolyGamma[2, 1 - lam] \
         - lam*((-1 + 2*EulerGamma*lam)*Pi^2 + 6*lam*Zeta[3])/3
      ) \
      + HarmonicSum[{1}, -lam]*(
         + 2*lam^2*HarmonicSum[{2, 2}, -lam] \
         + 4*lam^3*HarmonicSum[{3, 2}, -lam] \
         + lam^2*Pi^2*PolyGamma[1, 1 - lam]/3 \
         - lam^3*Pi^2*PolyGamma[2, 1 - lam]/3 \
         + lam^2*PolyGamma[3, 1 - lam]/3 \
         - lam^3*PolyGamma[4, 1 - lam]/6 \
         + lam^2*((-7*Pi^4)/180 - 2*lam*Pi^2*Zeta[3] + 18*lam*Zeta[5])
      ) \
      + lam*(
         -(7 + 6*EulerGamma*lam)*Pi^4/180 + 4*lam^2*Pi^6/945 \
         + 20*EulerGamma*lam^2*Pi^2*Zeta[3]/3 \
         + lam*(Zeta[5] - 3*lam*(Zeta[3]^2 + 28*EulerGamma*Zeta[5]))
      ) \
      + NSum[
         (-2*((1 + n)*(1 - 3*lam + 3*lam^2 + (2 - 3*lam)*n + n^2) +
         lam*(1 - lam + n)^3*PolyGamma[0, 1 - lam])*PolyGamma[0, 1 + n])/(1 - lam + n)^5,
         {n, 0, Infinity}
      ] \
      + sum4[lam]
   );

   sum4[lam_] = ApproxSum[
      (-2*PolyGamma[0, 1 - lam + n]*(-1 + 3*lam - 4*lam^2 - 3*EulerGamma*lam^2 - 3*lam^3 +
      2*EulerGamma*lam^3 - lam^4 + EulerGamma*lam^4 - 4*n + 9*lam*n - 8*lam^2*n -
      9*EulerGamma*lam^2*n - 3*lam^3*n + 4*EulerGamma*lam^3*n + EulerGamma*lam^4*n -
      6*n^2 + 9*lam*n^2 - 4*lam^2*n^2 - 9*EulerGamma*lam^2*n^2 + 2*EulerGamma*lam^3*n^2 -
      4*n^3 + 3*lam*n^3 - 3*EulerGamma*lam^2*n^3 - n^4 -
      lam*(1 + n)*(1 - 4*lam + lam^2 + 2*lam^3 + (3 - 8*lam + lam^2)*n + (3 - 4*lam)*n^2 +
      n^3)*PolyGamma[0, 1 - lam] - lam^3*(1 - lam + n)^2*(1 + lam + n)*
      PolyGamma[1, 1 - lam] + lam^2*PolyGamma[1, 1 - lam + n] -
      2*lam^4*PolyGamma[1, 1 - lam + n] + lam^6*PolyGamma[1, 1 - lam + n] +
      4*lam^2*n*PolyGamma[1, 1 - lam + n] - 4*lam^4*n*PolyGamma[1, 1 - lam + n] +
      6*lam^2*n^2*PolyGamma[1, 1 - lam + n] - 2*lam^4*n^2*PolyGamma[1, 1 - lam + n] +
      4*lam^2*n^3*PolyGamma[1, 1 - lam + n] + lam^2*n^4*PolyGamma[1, 1 - lam + n]))/
      ((1 + n)*(1 - lam + n)^5),
      lam,
      {n, 0, Infinity},
      CutOff -> (If[Abs[#] < 3,1000,Infinity] &),
      ExpansionOrder -> 7,
      AsymptoticPrecision -> 50
   ];

];

(* ********** original formula (for reference) **********
 * Ins[lam_, {1, 1, 1}, {0, 1, 1}, Sum1] := -2*N[Sum[(PolyGamma[0, 1 + \
 * n]*((1 + n)*(1 + n^2 + n*(2 - 3*lam) - 3*lam + 3*lam^2) + \
 * (1 + n - lam)^3*lam*PolyGamma[0, 1 - lam]))/(1 + n - \
 * lam)^5, {n, 0, Infinity}]] -
 *    2*N[Sum[(1/((1 + n)*(1 + n - lam)^5))*(PolyGamma[0, 1 + n - \
 * lam]*(-1 - 4*n - 6*n^2 - 4*n^3 - n^4 + 3*lam + 9*n*lam + \
 * 9*n^2*lam + 3*n^3*lam - 4*lam^2 - 3*EulerGamma*lam^2 \
 * - 8*n*lam^2 - 9*EulerGamma*n*lam^2 - 4*n^2*lam^2 - \
 * 9*EulerGamma*n^2*lam^2 -
 *          3*EulerGamma*n^3*lam^2 - 3*lam^3 + \
 * 2*EulerGamma*lam^3 - 3*n*lam^3 + 4*EulerGamma*n*lam^3 + \
 * 2*EulerGamma*n^2*lam^3 - lam^4 + EulerGamma*lam^4 + \
 * EulerGamma*n*lam^4 -
 *          (1 + n)*lam*(1 + n^3 + n^2*(3 - 4*lam) - 4*lam + \
 * lam^2 + 2*lam^3 + n*(3 - 8*lam + lam^2))*PolyGamma[0, \
 * 1 - lam] - (1 + n - lam)^2*lam^3*(1 + n + \
 * lam)*PolyGamma[1, 1 - lam] + lam^2*PolyGamma[1, 1 + n - \
 * lam] + 4*n*lam^2*PolyGamma[1, 1 + n - lam] +
 *          6*n^2*lam^2*PolyGamma[1, 1 + n - lam] + \
 * 4*n^3*lam^2*PolyGamma[1, 1 + n - lam] + \
 * n^4*lam^2*PolyGamma[1, 1 + n - lam] - 2*lam^4*PolyGamma[1, \
 * 1 + n - lam] - 4*n*lam^4*PolyGamma[1, 1 + n - lam] - 2*n^2*\
 * lam^4*PolyGamma[1, 1 + n - lam] +
 *          lam^6*PolyGamma[1, 1 + n - lam])), {n, 0, Infinity}]] +
 *    lam*(-2*N[Sum[((-1 + 2*lam + EulerGamma*lam + \
 * lam^2 - EulerGamma*lam^3 + n^2*(-1 + EulerGamma*lam) + \
 * 2*n*(-1 + lam + EulerGamma*lam) + lam*(-1 - 2*n - n^2 + \
 * lam^2)*PolyGamma[0, 1 - lam])*PolyGamma[1, 1 + n - lam])/(1 \
 * + n - lam)^4, {n, 0, Infinity}]] +
 *      4*lam*NSum[((1 + n + lam)*PolyGamma[0, 1 + n - \
 * lam]^2)/(1 + n - lam)^4, {n, 0, Infinity}] + lam*N[Sum[((1 \
 * + n)*PolyGamma[2, 2 + n - lam])/(1 + n - lam)^3, {n, 0, Infinity}]]) \
 * +
 *    (1/60)*((Plus[lam^2])*(-60*(EulerGamma - PolyGamma[0, 1 - \
 * lam])*(EulerGamma + lam*PolyGamma[1, 1 - lam])*PolyGamma[2, \
 * 1 - lam] + 20*(EulerGamma + EulerGamma^2*lam - 2*EulerGamma*\
 * lam*PolyGamma[0, 1 - lam] + lam*PolyGamma[0, 1 - \
 * lam]^2)*
 *        PolyGamma[3, 1 - lam] + 5*PolyGamma[4, 1 - lam] - \
 * 15*EulerGamma*lam*PolyGamma[4, 1 - lam] + \
 * 10*lam*PolyGamma[0, 1 - lam]*PolyGamma[4, 1 - lam] + 4*\
 * lam*PolyGamma[5, 1 - lam]))
 **********************************************************)

Ins[lam_, {1, 1, 1}, {0, 1, 1}, Sum2] := (
   + 11/5*Zeta[2]^2 + 2*Zeta[2]*Zeta[3]*lam - Zeta[2]*PolyGamma[1, 1 - lam] \
   + Zeta[2]*PolyGamma[2, 1 - lam]*lam + 3*Zeta[5]*lam \
   - 1/2*PolyGamma[1, 1 - lam]*PolyGamma[2, 1 - lam]*lam \
   + 1/2*PolyGamma[2, 1 - lam]*HarmonicSum[{1}, -lam] \
   - 1/2*PolyGamma[3, 1 - lam] \
   - 1/3*PolyGamma[3, 1 - lam]*HarmonicSum[{1}, -lam]*lam \
   + 1/8*PolyGamma[4, 1 - lam]*lam - HarmonicSum[{2, 2}, -lam] \
   - 3*HarmonicSum[{3, 1}, -lam] - 2*HarmonicSum[{3, 2}, -lam]*lam \
   - 4*HarmonicSum[{4, 1}, -lam]*lam
);

(* ********** original formula (for reference) **********
 * Ins[lam_, {1, 1, 1}, {0, 1, 1}, Sum2] := (
 *    -NSum[
 *       ((1 + n + lam)*PolyGamma[1, 1 + n - lam])/(1 + n - lam)^3,
 *       {n, 0, Infinity}
 *    ] +
 *    NSum[
 *       ((3 + 3*n + lam)*PolyGamma[0, 1 + n - lam])/(1 + n - lam)^4,
 *       {n, 0, Infinity}
 *    ]
 * ) + (1/24)*(
 *    -12*(2*EulerGamma - PolyGamma[0, 1 - lam] + lam*PolyGamma[1, 1 - lam])*PolyGamma[2, 1 - lam] +
 *    4*PolyGamma[3, 1 - lam] + 8*EulerGamma*lam*PolyGamma[3, 1 - lam] -
 *    8*lam*PolyGamma[0, 1 - lam]*PolyGamma[3, 1 - lam] - 3*lam*PolyGamma[4, 1 - lam]
 * )
 **********************************************************)

Module[
   {singlesum,doublesum,compileLib},

   Ins[lam_, {1, 1, 1}, {1, 1, 1}, Sum1] := singlesum[lam] + doublesum[lam,20000,40000];

   singlesum[lam_] = ApproxSum[
      -(((-1+EulerGamma+EulerGamma*n)*(EulerGamma*(1+n)*(1+n-lam)+lam)^2)/((1+n)^2*(1+n-lam)^6))+
      (2*lam^2*PolyGamma[0,1-lam]^3)/((1+n)*(1+n-lam)^4)-((1+n+lam)*(2+2*n^4-6*lam+(5+3*EulerGamma)*lam^2+
      (2-4*EulerGamma)*lam^3+(1+EulerGamma)*lam^4+n^3*(8-6*lam+3*EulerGamma*lam^2)+
      n^2*(12-18*lam+(5+9*EulerGamma)*lam^2-4*EulerGamma*lam^3)+n*(8-18*lam+(10+9*EulerGamma)*lam^2+
      (2-8*EulerGamma)*lam^3+EulerGamma*lam^4))*PolyGamma[0,1+n-lam]^2)/((1+n)^2*(1+n-lam)^5*lam^2)+
      PolyGamma[0,1+n]*(-((EulerGamma*(1+n)*(1+n-lam)+lam)^2/((1+n)*(1+n-lam)^6))-
      (lam^2*PolyGamma[0,1-lam]^2)/((1+n)*(1+n-lam)^4)+(2*(1+n+lam)*PolyGamma[0,1+n-lam])/((1+n)*(1+n-lam)^2*lam^2)+
      ((1+n+lam)^2*PolyGamma[0,1+n-lam]^2)/((1+n)*(1+n-lam)^4)+
      PolyGamma[0,1-lam]*((2*lam*(EulerGamma*(1+n)*(1+n-lam)+lam))/((1+n)*(1+n-lam)^5)+
      (2*(1+n+lam)*PolyGamma[0,1+n-lam])/((1+n)*(1+n-lam)^2*lam)))-(2*lam*(EulerGamma^2*(1+n)*(1+n-lam)^2+lam+
      EulerGamma*(1+2*n+n^2-lam^2))*PolyGamma[1,1-lam])/((1+n)*(1+n-lam)^5)+(2*lam*(EulerGamma^2*(1+n)*(1+n-lam)^2+lam+
      EulerGamma*(1+2*n+n^2-lam^2))*PolyGamma[1,1+n-lam])/((1+n)*(1+n-lam)^5)+PolyGamma[0,1-lam]^2*((lam*(-(lam*(3+3*n+lam))-
      EulerGamma*(1+n)*(4+4*n^2+n*(8-3*lam)-3*lam-lam^2)))/((1+n)^2*(1+n-lam)^5)-
      (2*lam*(1+n+2*lam)*PolyGamma[0,1+n-lam])/((1+n)*(1+n-lam)^4)-
      (2*lam^2*PolyGamma[1,1-lam])/((1+n)*(1+n-lam)^3)+(2*lam^2*PolyGamma[1,1+n-lam])/((1+n)*(1+n-lam)^3))+
      (lam*(EulerGamma*(1+n)*(1+n-lam)+lam)*PolyGamma[2,1-lam])/((1+n)*(1+n-lam)^4)-
      (lam*(EulerGamma*(1+n)*(1+n-lam)+lam)*PolyGamma[2,1+n-lam])/((1+n)*(1+n-lam)^4)+
      PolyGamma[0,1-lam]*((2*(lam^3+EulerGamma^2*(1+n)^2*(1+n-lam)^2*(1+n+lam)+
      EulerGamma*(1+n)*lam*(1+n^2+lam-2*lam^2+n*(2+lam))))/((1+n)^2*(1+n-lam)^6)+
      (2*(1+n+lam)*(-1-n+2*lam+lam^2/(1+n))*PolyGamma[0,1+n-lam]^2)/((1+n-lam)^4*lam)+
      (2*lam*(2*lam+EulerGamma*(1+2*n+n^2-lam^2))*PolyGamma[1,1-lam])/((1+n)*(1+n-lam)^4)-
      (2*lam*(2*lam+EulerGamma*(1+2*n+n^2-lam^2))*PolyGamma[1,1+n-lam])/((1+n)*(1+n-lam)^4)+
      PolyGamma[0,1+n-lam]*((2*(lam*(3+3*n+5*lam)+
      EulerGamma*(1+n^3+3*lam-3*lam^2-lam^3+3*n^2*(1+lam)+n*(3+6*lam-3*lam^2))))/((1+n)*(1+n-lam)^5)+
      (2*lam*(1+n+lam)*PolyGamma[1,1-lam])/((1+n)*(1+n-lam)^3)-(2*(1+n+lam)^2*PolyGamma[1,1+n-lam])/((1+n)*(1+n-lam)^3))-
      (lam^2*PolyGamma[2,1-lam])/((1+n)*(1+n-lam)^3)+(lam^2*PolyGamma[2,1+n-lam])/((1+n)*(1+n-lam)^3))+
      PolyGamma[0,1+n-lam]*((-2*(EulerGamma^2*(1+n)*(1+n-lam)^2*(2+2*n+lam)+lam*(3+3*n+4*lam)+
      EulerGamma*(1+n^3+4*lam-3*lam^2-2*lam^3+n^2*(3+4*lam)+n*(3+8*lam-3*lam^2))))/((1+n)*(1+n-lam)^6)-
      (2*(1+EulerGamma*(1+n-lam))*lam*(1+n+lam)*PolyGamma[1,1-lam])/((1+n)*(1+n-lam)^4)+
      (2*(1+n+lam)*(-1+(2+EulerGamma)*lam+lam^2-EulerGamma*lam^3+n^2*(-1+EulerGamma*lam)+
      2*n*(-1+lam+EulerGamma*lam))*PolyGamma[1,1+n-lam])/((1+n)*(1+n-lam)^4*lam)+
      (lam*(1+n+lam)*PolyGamma[2,1-lam])/((1+n)*(1+n-lam)^3)-
      ((1+n+lam)^2*PolyGamma[2,1+n-lam])/((1+n)*(1+n-lam)^3)),
      lam,
      {n,0,Infinity},
      ExpansionOrder -> 7,
      CutOff -> (Max[500,10*Abs[#]]&),
      AsymptoticPrecision -> 50
   ];

   doublesum[lam_?NumberQ,_Integer,_Integer] := NSum[
      (2*(1+m+lam)*(1+n+lam)*PolyGamma[0,1+m-lam]*PolyGamma[0,1+n-lam])/
      ((m-n)*(1+n)*(1+m-lam)^2*(1+n-lam)^2)
      ,{m,0,Infinity}
      ,{n,m+1,Infinity}
   ];
];

Ins[lam_, {1, 1, 1}, {1, 1, 1}, Sum2] = ApproxSum[
   -(-2*(2*EulerGamma*(1+n-lam)^2+EulerGamma^2*(1+n)*(1+n-lam)^2+(2*(1+n-lam)*lam)/(1+n)-lam^2/(1+n)+
   2*EulerGamma*(1+n)*(1+n-lam)^2*PolyGamma[0,1+n]+2*(1+n-lam)*lam*PolyGamma[0,1+n]-
   ((1+n-lam)^2*(lam*PolyGamma[0,1-lam]-(1+n+lam)*PolyGamma[0,1+n-lam])^2)/(1+n)+
   (2*(1+n-lam)^2*(-(lam*PolyGamma[0,1-lam])+(1+n+lam)*PolyGamma[0,1+n-lam]))/(1+n)-
   (2*(1+n-lam)*lam*(-(lam*PolyGamma[0,1-lam])+(1+n+lam)*PolyGamma[0,1+n-lam]))/(1+n)+
   2*(1+n-lam)^2*PolyGamma[0,1+n]*(-(lam*PolyGamma[0,1-lam])+(1+n+lam)*PolyGamma[0,1+n-lam]))+
   (1+n-lam)*(4*EulerGamma*(EulerGamma*(1+n)*(1+n-lam)+lam)+
   4*(EulerGamma*(1+n)*(1+n-lam)+lam)*PolyGamma[0,1+n]-
   4*(lam+EulerGamma*(1+2*n+n^2-lam^2))*PolyGamma[0,1-lam]-
   4*(1+n-lam)*lam*PolyGamma[0,1+n]*PolyGamma[0,1-lam]+4*(1+n-lam)*lam*PolyGamma[0,1-lam]^2+
   4*(1+n+2*lam+EulerGamma*(2+2*n^2-n*(-4+lam)-lam-lam^2))*PolyGamma[0,1+n-lam]+
   2*(1+n-lam)*(1+n+lam)*PolyGamma[0,1+n]*PolyGamma[0,1+n-lam]-
   2*(1+n-lam)*(1+n+3*lam)*PolyGamma[0,1-lam]*PolyGamma[0,1+n-lam]+
   2*(1+n-lam)*(1+n+lam)*PolyGamma[0,1+n-lam]^2+
   (2*(1+n-lam)*(1+n+lam)*PolyGamma[0,1+n-lam]^2)/(1+n)+
   (2*n*(1+n-lam)*(1+n+lam)*PolyGamma[0,1+n-lam]^2)/(1+n)+
   (2*(1+n-lam)*lam*(1+n+2*EulerGamma*(1+n)*(1+n-lam)+lam)*PolyGamma[1,1-lam])/(1+n)-
   (2*(1+n-lam)^2*lam*(1+n+lam)*PolyGamma[0,1-lam]*PolyGamma[1,1-lam])/(1+n)+
   (2*(1+n-lam)^2*lam*(1+n+lam)*PolyGamma[0,1+n-lam]*PolyGamma[1,1-lam])/(1+n)-
   (2*(1+n-lam)*lam*(1+n+2*EulerGamma*(1+n)*(1+n-lam)+lam)*PolyGamma[1,1+n-lam])/(1+n)+
   (2*(1+n-lam)^2*lam*(1+n+lam)*PolyGamma[0,1-lam]*PolyGamma[1,1+n-lam])/(1+n)-
   (2*(1+n-lam)^2*(1+n+lam)^2*PolyGamma[0,1+n-lam]*PolyGamma[1,1+n-lam])/(1+n)-
   (1+n-lam)^2*lam*PolyGamma[2,1-lam]+(1+n-lam)^2*lam*PolyGamma[2,1+n-lam])+
   (1+n-lam)*(4*EulerGamma*(EulerGamma*(1+n)*(1+n-lam)+lam)+
   4*(EulerGamma*(1+n)*(1+n-lam)+lam)*PolyGamma[0,1+n]-
   4*(lam+EulerGamma*(1+2*n+n^2-lam^2))*PolyGamma[0,1-lam]-
   4*(1+n-lam)*lam*PolyGamma[0,1+n]*PolyGamma[0,1-lam]+
   4*(1+n-lam)*lam*PolyGamma[0,1-lam]^2+4*(1+n+2*lam+EulerGamma*(2+2*n^2-n*(-4+lam)-lam-lam^2))*PolyGamma[0,1+n-lam]+
   2*(1+n-lam)*(1+n+lam)*PolyGamma[0,1+n]*PolyGamma[0,1+n-lam]-2*(1+n-lam)*(1+n+3*lam)*PolyGamma[0,1-lam]*PolyGamma[0,1+n-lam]+
   2*(1+n-lam)*PolyGamma[0,1+n-lam]^2+2*n*(1+n-lam)*PolyGamma[0,1+n-lam]^2+2*(1+n-lam)*lam*PolyGamma[0,1+n-lam]^2+
   2*(1+n+lam)*PolyGamma[0,1+n-lam]^2+2*n*(1+n+lam)*PolyGamma[0,1+n-lam]^2-2*lam*(1+n+lam)*PolyGamma[0,1+n-lam]^2+
   (2*(1+n-lam)*lam*(1+n+2*EulerGamma*(1+n)*(1+n-lam)+lam)*PolyGamma[1,1-lam])/(1+n)-
   (2*(1+n-lam)^2*lam*(1+n+lam)*PolyGamma[0,1-lam]*PolyGamma[1,1-lam])/(1+n)+
   (2*(1+n-lam)^2*lam*(1+n+lam)*PolyGamma[0,1+n-lam]*PolyGamma[1,1-lam])/(1+n)-
   (2*(1+n-lam)*lam*(1+n+2*EulerGamma*(1+n)*(1+n-lam)+lam)*PolyGamma[1,1+n-lam])/(1+n)+
   (2*(1+n-lam)^2*lam*(1+n+lam)*PolyGamma[0,1-lam]*PolyGamma[1,1+n-lam])/(1+n)-
   (2*(1+n-lam)^2*(1+n+lam)^2*PolyGamma[0,1+n-lam]*PolyGamma[1,1+n-lam])/(1+n)-
   (1+n-lam)^2*lam*PolyGamma[2,1-lam]+(1+n-lam)^2*lam*PolyGamma[2,1+n-lam]))/(2*(1+n-lam)^6),
   lam,
   {n,0,Infinity},
   ExpansionOrder -> 9,
   CutOff -> (Max[40,7*Abs[#]]&),
   AsymptoticPrecision -> 50
];

Ins[lam_, {1, 1, 1}, {1, 1, 1}, Sum3] := (
   -27/10*Zeta[2]^2 + Zeta[2]*PolyGamma[1, 1 - lam] \
   - Zeta[2]*PolyGamma[2, 1 - lam]*lam - 9*Zeta[5]*lam \
   + 1/2*PolyGamma[1, 1 - lam]*PolyGamma[2, 1 - lam]*lam \
   - 1/2*PolyGamma[2, 1 - lam]*HarmonicSum[{1}, -lam] \
   + 2/3*PolyGamma[3, 1 - lam] \
   + 1/2*PolyGamma[3, 1 - lam]*HarmonicSum[{1}, -lam]*lam \
   - 1/6*PolyGamma[4, 1 - lam]*lam + HarmonicSum[{2, 2}, -lam] \
   + 4*HarmonicSum[{3, 1}, -lam] + 2*HarmonicSum[{3, 2}, -lam]*lam \
   + 6*HarmonicSum[{4, 1}, -lam]*lam
);

(* ********** original formula (for reference) **********
 * Ins[lam_,{1,1,1},{1,1,1},Sum3] := Sum[
 *    (1/(1 + n - lam)^5)*(1 - 2*EulerGamma + n - 4*EulerGamma*n - 2*EulerGamma*n^2 - 3*lam +
 *    2*EulerGamma*lam + 2*EulerGamma*n*lam + (1 + n^2 + lam - 2*lam^2 + n*(2 + lam))*PolyGamma[0, 1 - lam] +
 *    (-3 - 3*n^2 + n*(-6 + lam) + lam + 2*lam^2)*PolyGamma[0, 1 + n - lam] -
 *    lam*PolyGamma[1, 1 - lam] - 2*n*lam*PolyGamma[1, 1 - lam] -n^2*lam*PolyGamma[1, 1 - lam] +
 *    2*lam^2*PolyGamma[1, 1 - lam] + 2*n*lam^2*PolyGamma[1, 1 - lam] -
 *    lam^3*PolyGamma[1, 1 - lam] + lam*PolyGamma[1, 1 + n - lam] + 2*n*lam*PolyGamma[1, 1 + n - lam] +
 *    n^2*lam*PolyGamma[1, 1 + n - lam] - 2*lam^2*PolyGamma[1, 1 + n - lam] - 2*n*lam^2*PolyGamma[1, 1 + n - lam] +
 *    lam^3*PolyGamma[1, 1 + n - lam]) +
 *    ((1 + n)*(-EulerGamma - PolyGamma[0, 2 + n - lam] + (1 + n - lam)*PolyGamma[1, 2 + n - lam]))/(1 + n - lam)^4,
 *    {n, 0, Infinity}
 * ]
  **********************************************************)

Ins[lam_, {1, 1/2}, {1, 0}, Sum1] := (
   - 8*Zeta[3]*lam + 2*PolyGamma[1,1-lam]*lam - PolyGamma[2,1-lam]*lam \
   - 2*HarmonicSum[{1},-lam] + 2*HarmonicSum[{1},-lam]^2 \
   + 4*HarmonicSum[{2,1},-lam]*lam
);

(* ********** original formula (for reference) **********
 * Ins[lam_, {1, 1/2}, {1, 0}, Sum1] := (
 *    -2*EulerGamma + 2*EulerGamma^2 -
 *    2*NSum[
 *       (2*lam*(EulerGamma + PolyGamma[0, k - lam]))/(k - lam)^2,
 *       {k, 1, Infinity}
 *    ] +
 *    (-2 + 4*EulerGamma)*PolyGamma[0, 1 - lam] +
 *    2*PolyGamma[0, 1 - lam]^2 + 2*lam*PolyGamma[1, 1 - lam] +
 *    lam*PolyGamma[2, 1 - lam]
 * );
 **********************************************************)

Module[
   {numsum},

   Ins[lam_, {1, 1/2}, {1, 0}, Sum2] := (1/lam^2)*(
      12*EulerGamma + 12*EulerGamma^2*lam - 6*EulerGamma^2*lam^2 +
      4*EulerGamma^3*lam^2 + Pi^2*lam^2  + 12*PolyGamma[0, 1 - lam] +
      36*EulerGamma*lam*PolyGamma[0, 1 - lam] -
      12*EulerGamma*lam^2*PolyGamma[0, 1 - lam] +
      12*EulerGamma^2*lam^2*PolyGamma[0, 1 - lam] +
      2*Pi^2*lam^2*PolyGamma[0, 1 - lam] + 24*lam*PolyGamma[0, 1 - lam]^2 -
      6*lam^2*PolyGamma[0, 1 - lam]^2 +
      24*EulerGamma*lam^2*PolyGamma[0, 1 - lam]^2 +
      16*lam^2*PolyGamma[0, 1 - lam]^3 +
      12*lam*PolyGamma[1, 1 - lam] - 6*lam^2*PolyGamma[1, 1 - lam]
      + 24*EulerGamma*lam^2*PolyGamma[1, 1 - lam] -
      12*EulerGamma*lam^3*PolyGamma[1, 1 - lam] +
      12*EulerGamma^2*lam^3*PolyGamma[1, 1 - lam] +
      24*lam^2*PolyGamma[0, 1 - lam]*PolyGamma[1, 1 - lam] -
      12*lam^3*PolyGamma[0, 1 - lam]*PolyGamma[1, 1 - lam] +
      24*EulerGamma*lam^3*PolyGamma[0, 1 - lam]*PolyGamma[1, 1 - lam] +
      12*lam^3*PolyGamma[0, 1 - lam]^2*PolyGamma[1, 1 - lam] +
      4*lam^2*PolyGamma[2, 1 - lam] - 6*lam^3*PolyGamma[2, 1 - lam] -
      6*EulerGamma*lam^3*PolyGamma[2, 1 - lam] +
      6*lam^3*PolyGamma[0, 1 - lam]*PolyGamma[2, 1 - lam] +
      8*lam^2*Zeta[3]- 12*lam^2*numsum[lam]
   );

   numsum[lam_] = ApproxSum[
      (EulerGamma*(k+lam-2*k*lam+2*EulerGamma*k*lam)+
      (k+2*lam-2*k*lam+2*EulerGamma*k*lam+2*lam^2*PolyGamma[0,1-lam])*PolyGamma[0,k-lam]+
      (k^2-lam^2)*PolyGamma[0,k-lam]^2+
      k*PolyGamma[0,k]*(2*EulerGamma*lam+(-k+lam)*PolyGamma[0,k-lam]))/(k*(k-lam)^2),
      lam,
      {k,1,Infinity},
      ExpansionOrder -> 7,
      CutOff -> (Max[200,10*Abs[#]] &),
      AsymptoticPrecision -> 50
   ];
];

Ins::nan = "Failed to compute Ins[`1`, {1/2}, {1}, Sum1]; shifting argument by `2`";
Ins::MaxIt = "Failed to compute Ins[`1`, {1/2}, {1}, Sum1] " <>
   " after `2` successive shifts of the argument";

Ins[lam_?NumberQ, {1/2}, {1}, Sum1] := Module[
   {
   shiftsize=10^-9,
   MaxIt=7,
   result,shift,numsum,lambda
   },
   numsum[lambda_] := NSum[
      (PolyGamma[0, n] - PolyGamma[0, n - lambda])*(
      PolyGamma[0, n]*PolyGamma[0, n - lambda] -
      PolyGamma[0, n - lambda]^2 - PolyGamma[1, n] +
      PolyGamma[1, n - lambda]),
      {n, 1, Infinity}
   ];
   result = numsum[lam];
   If[NumberQ[result], Return[result]];
   (* Mathematica can't solve the sum, shift argument by a tiny bit and retry *)
   Do[
      shift = shiftsize*Abs[lam]*E^(I*RandomReal[2*Pi]);
      Message[Ins::nan,lam,shift];
      result = numsum[lam+shift];
      If[NumberQ[result],
         (* use mean to improve precision *)
         result = Mean[{result,numsum[lam-shift]}];
         If[NumberQ[result],Return[result]];
      ];
      ,{MaxIt}
   ];
   If[!NumberQ[result], Message[Ins::MaxIt,InputForm[lam],MaxIt]];
   Return[result];
];

Ins[lam_, {1, del}, {1, 0}, Sum1] := (
   -18*HarmonicSum[{1}, -lam]*(1 + lam^2*PolyGamma[2, 1 - lam]) \
   + lam*(
      - lam*Pi^4 - 72*Zeta[3] + 36*HarmonicSum[{2, 1}, -lam] \
      + 72*lam*HarmonicSum[{3, 1}, -lam] + 18*PolyGamma[1, 1 - lam] \
      - 18*PolyGamma[2, 1 - lam] + 6*lam*PolyGamma[3, 1 - lam] \
   )
)/(36*lam);

(* ********** original formula (for reference) **********
 * Ins[lam_, {1, del}, {1, 0}, Sum1] := NSum[
 *    (EulerGamma*n^2 + lam - EulerGamma*n*lam + lam*(-n + lam)*PolyGamma[0, 1 - lam] + (n^2 - lam^2)*PolyGamma[0, n - lam])/
 *    ((n - lam)^4*(-1 + n^2 - 2*n*lam + lam^2)),
 *    {n, 1, Infinity}
 * ]
 **********************************************************)

Module[
   {LogNum},

   Ins[lam_, {0}, {1}, Sum1] := LogNum[-(4/(9*lam^2))];

   LogNum[et_?NumberQ] := Module[
      {integrand,pt,integral,phi},
      integrand[pt_] = (
         -(18*et*pt*(I*et + Sqrt[-et]*pt)*Hypergeometric2F1[1, 1 - 2/(3*Sqrt[-et]),
         2 - 2/(3*Sqrt[-et]), -((Sqrt[-et] + I*pt)/(Sqrt[-et] - I*pt))] +
         9*et*pt*(2*Sqrt[-et]*pt + et*(-2*I + (3*I)*Sqrt[-et] + 3*pt))*
         Hypergeometric2F1[1, 2/(3*Sqrt[-et]), 1 + 2/(3*Sqrt[-et]),
         -((Sqrt[-et] + I*pt)/(Sqrt[-et] - I*pt))] +
         2*Sqrt[-et]*(9*et*((Sqrt[-et] - I*pt)/(Sqrt[-et] + I*pt))^(2/(3*Sqrt[-et]))*pt*
         (I*Sqrt[-et] + pt)*Gamma[2 - 2/(3*Sqrt[-et])]*Gamma[2/(3*Sqrt[-et])] +
         (et*(-2 + 3*Sqrt[-et] - (3*I)*pt) - (2*I)*Sqrt[-et]*pt)*
         (-4 + 8*EulerGamma + (3*I)*pt*Log[Sqrt[-et]/(Sqrt[-et] - I*pt)] -
         (3*I)*pt*Log[Sqrt[-et]/(Sqrt[-et] + I*pt)] - 8*Log[pt^(-1)] -
         8*Log[pt/(2*Sqrt[-et])] + 8*PolyGamma[0, 1 - 2/(3*Sqrt[-et])])))/
         (* the factor which is commented out is eliminated *)
         (* by changing the integration variable *)
         (18*(2*Sqrt[-et] + 3*et)*Pi*(Sqrt[-et] - I*pt)(* *(et - pt^2) *))
      );

      pt[phi_] = Sqrt[et]*Tanh[phi];
      integral = NIntegrate[
         integrand[pt[phi]],
         {phi,0,-I*Pi/2*E^(Pi/3*I),-I*Pi/2}
      ];
      integral /= Sqrt[et];
      Return[
         I*Im[integral] +
         1/27*(-11 + 12*Log[4] + 9*Log[-et])
      ];
   ];

];

Module[
   {sum1,sum2,sum3,sum4,NonLogNum,kc=200},

   Ins[lam_, {0}, {1}, Sum2] := (
      -1836 + 1320*EulerGamma + 80*Pi^2 + \
      240/lam^2 + 1140/lam + (35*Pi^2)/lam + 60*Log[64] - \
      120*EulerGamma*Log[64] +
      (720*sum1[lam])/lam^2 \
      - 1620*NonLogNum[-(4/(9*lam^2))] + 1320*PolyGamma[0, 1 - lam] - \
      120*Log[64]*PolyGamma[0, 1 - lam] +
      720*(EulerGamma + PolyGamma[0, 1 - lam]) + 90*Pi^2*(EulerGamma + \
      PolyGamma[0, 1 - lam]) + 180*Log[4]^2*(EulerGamma + PolyGamma[0, 1 - \
      lam]) + 720*Log[4]*(-1 + Log[8])*(EulerGamma + PolyGamma[0, 1 - \
      lam]) -
      90*Log[16]*Log[256]*(EulerGamma + PolyGamma[0, 1 - lam]) + (1/\
      lam)*(10*(72*EulerGamma^2 + 72*EulerGamma^3*lam + \
      Pi^2*lam*Log[4096] + 6*EulerGamma*(3*Pi^2*lam - \
      12*lam*Log[2]^2 + Log[4096]) +
      6*(12*EulerGamma + 12*EulerGamma^2*lam + Pi^2*lam - 12*\
      lam*Log[2]^2 + Log[4096])*PolyGamma[0, 1 - lam])) + (720*(-1 + \
      EulerGamma*lam + lam*PolyGamma[0, -lam])^2)/lam^2 +
      (720*(1 - EulerGamma*lam - lam*PolyGamma[0, \
      -lam])*(Pi^2/2 + (EulerGamma + PolyGamma[0, -lam])^2 - \
      PolyGamma[1, -lam]))/lam -
      720*lam*sum2[lam] -
      (1/lam)*(90*(4*EulerGamma + Pi^2*lam + Log[16] + \
      4*PolyGamma[0, 1 - lam] + 2*PolyGamma[0, 1/2 - lam/2] - \
      2*PolyGamma[0, 1 - lam/2] +
      2*Sqrt[Pi]*lam^2*sum3[lam] + \
      4*lam^2*sum4[lam])) + \
      772*Zeta[3] -
      (1/lam^2)*(20*(2*EulerGamma*(-9 + 18*lam*Log[2] + \
      lam^2*(-24 + 5*Pi^2 - 36*Log[2] + 36*Log[2]^2)) + 2*(-9 + \
      18*lam*Log[2] + lam^2*(-24 + 5*Pi^2 - 36*Log[2] + \
      36*Log[2]^2))*PolyGamma[0, 1 - lam] + 3*lam*(Pi^2*(-1 + \
      lam*Log[4]) - 6*lam*Zeta[3])))
   );

   sum1[lam_] = ApproxSum[
      ((k - lam)*(PolyGamma[0, k - lam] - PolyGamma[0, k]) +
      k*lam*PolyGamma[1, k])^2/k,
      lam,
      {k,1,Infinity},
      ExpansionOrder -> 15,
      CutOff -> (Max[100,2*Abs[#]]&),
      AsymptoticPrecision -> 50
   ];

   sum2[lam_?NumberQ] := NSum[
      ((1 + 2*EulerGamma*n)*PolyGamma[0, n] + n*PolyGamma[0, n]^2)/
      (n^2*(n - lam)),
      {n, 1, Infinity}
   ];

   sum3[lam_] := Sum[Sum[
         ((-1)^n*(2*k)!*Gamma[2 + k]*(-2 + Log[4] \
         - PolyGamma[0, k] - 2*PolyGamma[0, 2*k] + PolyGamma[0, 3/2 + k] + \
         PolyGamma[0, 1 + k - n] + PolyGamma[0, 1 + k + n]))/
         (2^(2*k)*(k^2*(n - lam)*(k - n)!*Gamma[3/2 + k]*Gamma[1 + k + n])),
         {n, 1, k}
      ],
      {k, 1, kc}
   ];

   sum4[lam_] := NApproxSum[
      ((-1)^k*Gamma[k]*Gamma[2 + k]*Gamma[-k + n])/
      (k*(1 + 2*k)*(n - lam)*Gamma[1 + k + n]),
      lam,
      {n, 1, Infinity},
      {k, 1, n - 1},
      CutOff -> (Max[Abs[#],200] &)
   ];

   NonLogNum[et_?NumberQ] := Module[
      {
         (* minimum distance to pole in integrand *)
         dmin = 100,
         integrand,pt,phi,ptsafe,phisafe,integral
      },

      integrand[pt_] = -1/8/Pi*(
         (
            -I*(et + pt^2)*Log[(Sqrt[-et] + I*pt)/(Sqrt[-et] - I*pt)] +
            2*Sqrt[-et]*pt*(1 + Log[1/(pt^2 - et)])
         )*
         ( If[Abs[pt] < 10^5,
            4/(3*(2*Sqrt[-et] + 3*et)*(pt^2 - et))*
            ( 3*et*((-I)*et + (2*Sqrt[-et] - I*pt)*pt)*
            ( (1 - 3*Sqrt[-et]/2)*(Sqrt[-et] + I*pt)/(Sqrt[-et] - I*pt)*
            Hypergeometric2F1[1, 2/3/Sqrt[-et], 1 + 2/3/Sqrt[-et],
            -((Sqrt[-et] + I*pt)/(Sqrt[-et] - I*pt))]
            + Gamma[2 - 2/3/Sqrt[-et]]*Gamma[2/3/Sqrt[-et]]*
            ((Sqrt[-et] - I*pt)/(Sqrt[-et] + I*pt))^(2/3/Sqrt[-et] - 1)
            ) +
            (*         Hypergeometric2F1[1, 1 - 2/3/Sqrt[-et], 2 - 2/3/Sqrt[-et],
            -((Sqrt[-et] - I*pt)/(Sqrt[-et] + I*pt))] + *)
            3*et*(I*et + (2*Sqrt[-et] + I*pt)*pt)*
            Hypergeometric2F1[1, 1 - 2/3/Sqrt[-et], 2 - 2/3/Sqrt[-et],
            -((Sqrt[-et] + I*pt)/(Sqrt[-et] - I*pt))] -
            I*(2*Sqrt[-et] + 3*et)*(et - pt^2)*
            (Log[Sqrt[-et]/(Sqrt[-et] - I*pt)] - Log[Sqrt[-et]/(Sqrt[-et] + I*pt)])
            ) -
            32*(1 - EulerGamma + Log[pt/(2*Sqrt[-et])] - PolyGamma[0, 1 - 2/3/Sqrt[-et]])/(9*pt),
            -32*Pi/27/pt^4
         ] -
         32*(Log[1/pt] - 1/2)/(9*pt)
         )
      );

      phi[pt_] = ArcTanh[pt/Sqrt[et]];
      pt[phi_] = Sqrt[et]*Tanh[phi];
      ptsafe = Re[Sqrt[et]]+Sqrt[Max[dmin^2-Im[Sqrt[et]]^2,0]];
      phisafe = phi[ptsafe];
      integral = Im[
         NIntegrate[
            integrand[pt[phi]],
            {phi,0,-I*Pi/2*E^(Pi/3*I),phisafe}
         ]/Sqrt[et]
      ] +
      NIntegrate[
         Im[integrand[pt]/(et-pt^2)],
         {pt,ptsafe,Infinity}
      ];
      Return[I*integral];
   ];

];

HypIns1Der[lam_?NumberQ, 1] := Module[
   {ep = 0.00001 + 0.00001*I},
   (Ins[lam, {1}, {2}, Sum1] - Ins[lam + ep, {1}, {2}, Sum1])/ep
];

(* HypIns2Der[lam_, 1] = - D[Ins[lam, {1, 1}, {0, 1}, Sum1],lam] *)
HypIns2Der[lam_, 1] := (
   - 27/10*Zeta[2]^2 + Zeta[2]*PolyGamma[1,1-lam] \
   - Zeta[2]*PolyGamma[2,1-lam]*lam - 9*Zeta[5]*lam \
   + 1/2*PolyGamma[1,1-lam]*PolyGamma[2,1-lam]*lam \
   - 1/2*PolyGamma[2,1-lam]*HarmonicSum[{1},-lam] \
   + 2/3*PolyGamma[3,1-lam] \
   + 1/2*PolyGamma[3,1-lam]*HarmonicSum[{1},-lam]*lam \
   - 1/6*PolyGamma[4,1-lam]*lam \
   + HarmonicSum[{2,2},-lam] + 4*HarmonicSum[{3,1},-lam] \
   + 2*HarmonicSum[{3,2},-lam]*lam + 6*HarmonicSum[{4,1},-lam]*lam \
);

HypIns3Der[lam_?NumberQ, 1] := Module[
   {ep = 1.*10^-7 + 1.*10^-7*I},
   (Ins[lam, {1, 1}, {1, 1}, Sum1] - Ins[lam + ep, {1, 1}, {1, 1}, Sum1])/ep
];

Ins[lam_, US, opts:OptionsPattern[]] := Module[
   {mt = OptionValue[QuarkMass], as = OptionValue[alphaS]},
   If[Im[-((mt*cF^2*as^2)/(4*lam^2)) /. cF -> 4/3 //N] < 0.01, 0,
      USConst[N[-((mt*cF^2*as^2)/(4*lam^2)) /. cF -> 4/3],opts
      ]
   ]
];

Unprotect[Ins,HypIns1Der,HypIns2Der,HypIns3Der];
End[];


Print[Text["QQbarGridCalc successfully loaded."]]

EndPackage[]
