BeginPackage["horner`"];
horner::usage = "horner[expr_] optimises expr "<>
"using a multivariate generalisation of the Horner scheme";

Options[horner] = {
   Ignore -> {}
}

Begin["Private`"];
horner[e_, opts:OptionsPattern[]] := Module[
   {expr=Expand[e],VarCounter,TermList,var,mask,x},
   (* Print[expr]; *)
   If[ Head[expr] =!= Plus, Return[expr]];
   VarCounter[_] = 0;
   (* determine the variable that appears in the most terms *)
   allvars = Flatten[VarsInTerm /@ (List @@ expr)];
   allvars = Select[allvars, !MemberQ[OptionValue[Ignore],#]&];
   If[allvars === {}, Return[expr]];
   var = Commonest[allvars,1][[1]];
   If[Count[allvars,var] < 2, Return[expr]];
   expr = expr /. Power[var,x_Integer] /; x>0 :> var*mask[var^(x-1)];
   (* collect for the chosen variable and recursively apply to coefficients *)
   Return[ Collect[expr,var,horner[# /. mask -> Identity, opts]&] ];
];

VarsInTerm[t_] := Module[
   {term,x},
   If[Head[t] === Times,
      term = List @@ t;
      ,(* else *)
      term = {t}
   ];
   term = Select[term,!NumberQ[#]&];
   Return[ term /. Power[x_,y_Integer] /; y>0 -> x ];
];

End[];

EndPackage[];
