BeginPackage["QQbarThreshold`"];
Begin["`Private`"];

(* Expressions from BJR2010 (Nucl. Phys. B840 (2010) 186-213). *)
(* Code used with permission of Bernd Jantzen. *)

NonresonantXSection[sqrts_, width_, x_, DeltaM_] := Module[
   {s = sqrts^2, y = (1 - DeltaM/m)^2},
   32*Pi^2*alpha^2/s*width/m*Sum[
      diagramh[i, s, x, y],
      {i, 1, 10}
   ]
];

Gammatmt[x_] = alpha/(16*sw2)*(1-x)^2*(1+2*x)/x;

diagramh[1, s_, x_, y_] := (
   (eU^2*CAA[s]-2*vt*eU*CAZ[s]+CZZ[s]*vt^2)*6*Sqrt[2]/Pi^2/Sqrt[1-y]
   +(CAA[s]*eU^2+CAZ[s]*(-2*eU*vt)+CZZ[s]*vt^2)*h["1V", x, y]
   +(CAZ[s]*2*eU*at+CZZ[s]*(-2*vt*at))*h["1VA", x, y]
   +CZZ[s]*at^2*h["1A", x, y]
);

diagramh[2, s_, x_, y_] := (
   (CAA[s]*2*eD*eU+CAZ[s]*(-2*(eD*vt+eU*(vb+ab)))+CZZ[s]*2*(vb+ab)*vt)*h["2V", x, y]
   +(CAZ[s]*(-2*at*eD)+CZZ[s]*2*(vb+ab)*at)*h["2A", x, y]
);

diagramh[3, s_, x_, y_] := (
   (CAA[s]*2*eU+CAZ[s]*(-2*vt-2*Sqrt[cw2/sw2]*eU)+CZZ[s]*2*Sqrt[cw2/sw2]*vt)*h["3V", x, y]
   +(CAZ[s]*2*at+CZZ[s]*(-2*Sqrt[cw2/sw2]*at))*h["3A", x, y]
);

diagramh[4, s_, x_, y_] := (
   (CA[s]*eU+CZ[s]*vt)*h["4V", x, y]
   +CZ[s]*at*h["4A", x, y]
);

diagramh[5, s_, x_, y_] := (
   1/sw2^2*h["5", x, y]
);

diagramh[6, s_, x_, y_] := (
   (CA[s]+CZ[s]*Sqrt[cw2/sw2])*h["6", x, y]
);

diagramh[7, s_, x_, y_] := (
   (CA[s]*eD+CZ[s]*(vb+ab))*h["7", x, y]
);

diagramh[8, s_, x_, y_] := (
   (CAA[s]*eD^2+CAZ[s]*(-2*eD*(vb+ab))+CZZ[s]*(vb+ab)^2)*h["8", x, y]
);

diagramh[9, s_, x_, y_] := (
   (CAA[s]*2*eD+CAZ[s]*(-2*(vb+ab+eD*Sqrt[cw2/sw2]))+CZZ[s]*2*Sqrt[cw2/sw2]*(vb+ab))*h["9", x, y]
);

diagramh[10, s_, x_, y_] := (
   (CAA[s]+CAZ[s]*(-2*Sqrt[cw2/sw2])+CZZ[s]*cw2/sw2)*h["10", x, y]
);

CAA[s_] := -eE^2*m^2/(4*s);
CAZ[s_] := eE*ve*m^2/(4*(s-mZ^2));
CZZ[s_] := -s*(ve^2+ae^2)*m^2/(4*(s-mZ^2)^2);
CA[s_] := eE*m^2/(sw2*s);
CZ[s_] := (ve+ae)*m^2/(sw2*(s-mZ^2));

h[index_String, x_?NumericQ, y_?NumericQ] := Module[
   {
      tmin = y,
      ep = 10^-12
   },
   If[tmin >= 1 - ep, Return[0]];
   If[tmin <= x*(1+ep), tmin = x*(1+ep)];
   Return[
      h[index, x, y] = integrateh[index, x, {tmin, 1 - ep}]
   ];
];

integrateh[index_String, args__] := If[
   MatchQ[index, "4V"| "4A" | "5" | "6" | "7"],
   integrate2[index, args],
   integrate1[index, args]
];

integrate1[index_String, x_?NumericQ, {tmin_?NumericQ, tmax_?NumericQ}] := With[
   {prec = 25},
   Quiet[
      NIntegrate[
         hIntegrand[index, t, SetPrecision[x, prec+5]],
         {t, SetPrecision[tmin, prec+5], tmax},
         PrecisionGoal -> 9,
         WorkingPrecision -> prec
      ],
      {NIntegrate::precw}
   ]
];

integrate2[index_String, x_?NumericQ, {tmin_?NumericQ, tmax_?NumericQ}] := With[
   {prec = 20},
   Quiet[
      NIntegrate[
         hIntegrand[index, t, z, SetPrecision[x, prec+5]],
         {t, SetPrecision[tmin, prec+5], tmax},
         {z, -1, 1},
         PrecisionGoal -> 8,
         WorkingPrecision -> prec
      ],
      {NIntegrate::precw}
   ]
];

hIntegrand["1V", t_,x_]=(3*Sqrt[2])/(Pi^2*(1-t)^(3/2))+(Sqrt[9-t]*(-27-73*t+3*t^2+t^3)*(t-x)^2*(t+2*x))/(64*Pi^2*(1-t)^(3/2)*t^2*(-1+x)^2*(1+2*x));


hIntegrand["1A", t_,x_]=-(Sqrt[9-t]*t*(27+4*t+t^2)*(-1+x/t)^2*(1+(2*x)/t))/(64*Pi^2*Sqrt[1-t]*(-1+x)^2*(1+2*x));


hIntegrand["1VA", t_,x_]=(Sqrt[9-t]*t*(-27+2*t+t^2)*(-1+x/t)^2*(1+(2*x)/t))/(64*Pi^2*Sqrt[1-t]*(-1+x)^2*(1+2*x));


hIntegrand["2V", t_,x_]=1/(4*Pi^2*(x-1)^2*(2*x+1))*(
-(t*(2*x^2+7*x-5)-x*(2*x^2+15*x+31))/(t-1)*fNR[t,x]
+Sqrt[(9-t)/(1-t)]*(t^5+4*t^4-t^3*(3*x^2+16*x+53)+2*t^2*x*(x^2-6*x-16)+t*x^2*(24*x+79)+6*x^3)/(16*t^2)
);


hIntegrand["2A", t_,x_]=1/(4*Pi^2*(x-1)^2*(2*x+1))*(
-(t*(2*x^2+5*x-3)-x*(2*x^2+13*x+17))/(t-1)*fNR[t,x]
+Sqrt[(9-t)/(1-t)]*(t^5+2*t^4-t^3*(3*x^2+16*x+27)+2*t^2*x*(x^2-3*x-16)+5*t*x^2*(4*x+13)-6*x^3)/(16*t^2)
);


hIntegrand["3V", t_,x_]=1/(8*Pi^2*(x-1)^2*x*(2*x+1))*(
-(t^3*x+t^2*(4*x^2-6*x+2)+t*(-2*x^3-29*x^2+2)+x*(4*x^3+36*x^2+x-13))/(t-1)*ftNR[t,x]
+1/(8*t^2)*Sqrt[(9-t)/(1-t)]*(t^5*x-4*t^4*(x+1)+t^3*(-3*x^3-24*x^2+19*x+4)+2*t^2*x*(x^3+2*x^2-42*x-48)+t*x^2*(24*x^2+63*x+92)+6*x^4)
);


hIntegrand["3A", t_,x_]=1/(8*Pi^2*(x-1)^2*x*(2*x+1))*(
(t^3*x+t^2*(4*x^2-6*x+2)+t*(-2*x^3-23*x^2+20*x+2)+x*(4*x^3+24*x^2-33*x+7))/(t-1)*ftNR[t,x]
-1/(8*t^2)*Sqrt[(9-t)/(1-t)]*(t^5*x-6*t^4*x+t^3*(-3*x^3-24*x^2+53*x+16)+2*t^2*x*(x^3+5*x^2-64*x+20)+t*x^2*(20*x^2+81*x-56)-6*x^4)
);


hIntegrand["4V", t_,z_,x_]=-3*x/(64*Pi^2*(1-x)^2*(2*x+1))*Sqrt[(9-t)/(1-t)]*(
1/2*(
-(t-x)*(t*(2*t*x+t-17*x-1)+23*x)/(6*t*x^2)
+(t^3*x+t^2*(2*x^2-5*x+2)+t*(x+2)-x*(86*x+13))/(3*Sqrt[(9-t)*(1-t)]*x^2)*ftNR[t,x]
)
+1/2*(t^3-5*t^2-25*t+93)*gNR[0,t,z,x]
+(t^4-4*t^3-106*t^2+604*t-495)/(2*Sqrt[(9-t)*(1-t)])*gNR[1,t,z,x]
+1/2*(-t^3+21*t^2-119*t+99)*gNR[2,t,z,x]
-1/2*(9-t)^(3/2)*(1-t)^(3/2)*gNR[3,t,z,x]
);


hIntegrand["4A", t_,z_,x_]=3*x/(64*Pi^2*(1-x)^2*(2*x+1))*Sqrt[(9-t)/(1-t)]*(
1/2*(
(t^3*x-t^2*(x^2+8*x+2)+t*x*(8*x-5)+7*x^2)/(3*t*x^2)
-(t^3*x+t^2*(2*x^2-5*x+2)+t*(21*x+2)+(7-30*x)*x)/(3*Sqrt[(9-t)*(1-t)]*x^2)*ftNR[t,x]
)
-1/2*(t-3)*(t-1)^2*gNR[0,t,z,x]
+(-t^4+4*t^3+74*t^2-284*t+207)/(2*Sqrt[(9-t)*(1-t)])*gNR[1,t,z,x]
+1/2*(t^3-21*t^2+119*t-99)*gNR[2,t,z,x]
+1/2*(9-t)^(3/2)*(1-t)^(3/2)*gNR[3,t,z,x]
);


hIntegrand["5", t_,z_,x_]=3*x/(1024*Pi^2*(1-x)^2*(2*x+1))*Sqrt[(9-t)*(1-t)]*(
1/2*(
-1/(6*t^2*x^3*(t^2*x+t*(4-6*x)+x*(4*x-3)))*(
t^5*x*(8*x+1)+t^4*(-8*x^3+6*x^2+39*x+4)+t^3*(-39*x^3+14*x^2+7*x+84)+t^2*x*(32*x^3-25*x^2+2*x-111)-t*x^2*(28*x^2+x-18)+3*(3-4*x)*x^3)
+4*((t-9)*x^2+(t-11)*x-6)/(3*x^3*Sqrt[(9-t)*(1-t)])*ftNR[t,x]
)
+4*(t+3)/x*gNR[0,t,z,x]
-4*Sqrt[(9-t)*(1-t)]/x*gNR[1,t,z,x]
-4*x*(t^2-8*t+15)*jNR[0,t,z,x]
-8*x*(t-4)*Sqrt[(9-t)*(1-t)]*jNR[1,t,z,x]
-4*x*(9-t)*(1-t)*jNR[2,t,z,x]
+8*(t^2*x+t*(2-4*x)-x-6)/x*jtNR[0,t,z,x]
+8*Sqrt[(9-t)*(1-t)]*(t*x+2)/x*jtNR[1,t,z,x]
);


hIntegrand["6", t_,z_,x_]=-1/(128*Pi^2*(x-1)^2*x^2*(2*x+1))*Sqrt[(9-t)*(1-t)]*(
1/2*(
1/(t^2*(t^2*x+t*(4-6*x)+x*(4*x-3)))*(
t^5*x*(6*x^2+7*x+1)+t^4*(-6*x^4-28*x^3+8*x^2+35*x+4)+t^3*(-19*x^4-233*x^3+x^2-25*x+84)+t^2*x*(40*x^4+284*x^3-69*x^2+46*x-111)+t*x^2*(-60*x^3+33*x^2-13*x+18)+3*(3-4*x)*x^3)
+2*((t^2-8*t+19)*x^2+2*(t+1)*x^3-2*(t-17)*x+24)/Sqrt[(9-t)*(1-t)]*ftNR[t,x]
)
-6*(t+3)*x^2*(3*x+2)*gNR[0,t,z,x]
+6*Sqrt[(9-t)*(1-t)]*x^2*((t+6)*x+2)*gNR[1,t,z,x]
-6*(9-t)*(1-t)*x^3*gNR[2,t,z,x]
-6*x^2*(t^2*x*(x+4)-2*t*(5*x^2+8*x-4)+33*x^2+20*x-24)*jtNR[0,t,z,x]
+12*Sqrt[(9-t)*(1-t)]*x^2*(-2*(t-2)*x+7*x^2-4)*jtNR[1,t,z,x]
-6*(9-t)*(1-t)*x^4*jtNR[2,t,z,x]
);


hIntegrand["7", t_,z_,x_]=3*x/(64*Pi^2*(1-x)^2*(2*x+1))*Sqrt[(9-t)*(1-t)]/(5-t)*(
1/2*(
-(t-5)*(t-x)*(2*t*x+t-15*x)/(6*t*x^2)
-(-t^3*x+t^2*(2*x^2+9*x-2)-3*t*(8*x^2+3*x-2)+x*(-16*x^2+18*x+17))/(3*Sqrt[(9-t)*(1-t)]*x^2)*ftNR[t,x]
-4*(t^2*x^2+t*(-8*x^2+x-1)+(-4*x^2+2*x+13)*x)/(3*Sqrt[(9-t)*(1-t)]*x^2)*fNR[t,x]
)
-1/2*(-t^3+t^2*(2*x+13)-5*t*(4*x+3)+66*x-45)*gNR[0,t,z,x]
+1/2*Sqrt[(9-t)*(1-t)]*((t-2)*t+28*x-39)*gNR[1,t,z,x]
-1/2*(9-t)*(1-t)*(t+2*x-11)*gNR[2,t,z,x]
-1/2*((9-t)*(1-t))^(3/2)*gNR[3,t,z,x]
-1/2*(t^3-t^2*(2*x+7)+t*(20*x+3)-66*x-45)*gtNR[0,t,z,x]
+1/2*Sqrt[(9-t)*(1-t)]*(t*(t+4)-7*(4*x+3))*gtNR[1,t,z,x]
-1/2*(9-t)*(1-t)*(t-2*x+1)*gtNR[2,t,z,x]
+1/2*((9-t)*(1-t))^(3/2)*gtNR[3,t,z,x]
);


hIntegrand["8", t_,x_]=1/(4*Pi^2*(x-1)^2*(2*x+1))*(
(2*t*x+t-2*x-9)*fNR[t,x]
+Sqrt[(9-t)*(1-t)]/(16*t^2*(t*(1-x)+x*(x+3)))*(t^5*(1-x)+t^4*(x^2-2*x+5)+t^3*(3*x^3+50*x^2+39*x-72)-t^2*x*(5*x^3+72*x^2+151*x+72)+t*x^2*(2*x^3+29*x^2+149*x+216)-6*x^3*(x^2+7*x+12))
);


hIntegrand["9", t_,x_]=1/(8*Pi^2*(x-1)^2*x*(2*x+1))*(
2*(t*(2*x^3+7*x^2-7*x+2)-x*(2*x^3+23*x^2+25*x+6))/(t-5)*fNR[t,x]
+(t^3*x-2*t^2*(2*x^2+5*x-1)+t*(-2*x^3+19*x^2+12*x-6)-x*(4*x^3+16*x^2-25*x+17))/(t-5)*ftNR[t,x]
+Sqrt[(9-t)*(1-t)]/(8*t^2)*(t^4*x-t^3*(3*x+4)+t^2*x*(-3*x^2+8*x+24)+t*x^2*(2*x^2+x+4)-6*x^3*(x+4))
);


hIntegrand["10", t_,x_]=1/(4*Pi^2*(x-1)^2*x^2*(2*x+1))*(
-((-t^2+7*t-3)*x^2+(4*t+11)*x^3+6*x^4-12*x-12)*ftNR[t,x]
+Sqrt[(9-t)*(1-t)]/(16*t^2*(t^2*x+t*(4-6*x)+x*(4*x-3)))*(t^6*x^3+t^5*x*(47*x^2+28*x+4)+t^4*(-3*x^5-28*x^4-249*x^3+60*x^2+124*x+16)+t^3*(2*x^6+67*x^5+480*x^4-747*x^3-112*x^2-228*x+336)-t^2*x*(94*x^5+205*x^4-884*x^3+220*x^2-360*x+444)+t*x^2*(8*x^5-30*x^4-123*x^3+136*x^2-100*x+72)-6*x^3*(4*x^4+13*x^3-12*x^2+8*x-6))
);

fNR[t_,x_] :=
Log[(Sqrt[(9-t)*(1-t)]*(x-t)-t*(t-5)+x*(t+3))/(Sqrt[(9-t)*(1-t)]*(t-x)-t*(t-5)+x*(t+3))];


ftNR[t_,x_] :=
Log[(Sqrt[(9-t)*(1-t)]*(x-t)-t*(t-5)-x*(t+3))/(Sqrt[(9-t)*(1-t)]*(t-x)-t*(t-5)-x*(t+3))];


scoeffs={
a->-4*(9-t)*(1-t)*(x-1),
b->4*Sqrt[(9-t)*(1-t)]*(x-2)*(t+2*x-3),
c->t^2*(x^2+4)-2*t*(5*x^2-8*x+12)+25*x^2-48*x+36,
d->-2*t^2+t*((x-1)*x+6)-x*(x+3),
e->-Sqrt[(9-t)*(1-t)]*(t*(x-2)+x),
k->t-x,
at->-1/4*(9-t)*(1-t)*(t^2-2*t-16*x+1),
bt->1/2*Sqrt[(t-9)*(t-1)]*((t-6)*t-8*x-3)*(t-2*x-1),
ct->1/4*(t^4-4*t^3*(x+3)+t^2*(4*x^2+44*x+30)-4*t*(10*x^2+35*x-9)+100*x^2+228*x+9),
dt->1/2*(-t^3+3*t^2*(x+2)+t*(-2*x^2-16*x+3)+x*(2*x+21)),
et->1/2*Sqrt[(t-9)*(t-1)]*(t*(-t+x+1)+7*x),
c1->x*((t-6)*t+4*x-3)/t+4,
c2->(4*zt*(t*(x-2)+x)-t^2*(x-4)-t*x*(x+3)+x^2)/t,
c3->x*(2*zt-t)*(2*zt-x)/t,
d1->((t-7)*(t-3)*t^2+(t*(t+2)+9)*x^2+(t*(t+2)-27)*t*x)/t^2,
d2->(4*zt*((t-3)*t^2+(t+3)*x^2+(t-3)*t*x)-t*(t^2*(3*x+4)+t*x*(3*x-23)+13*x^2))/t^2,
d3->(4*zt^2*(t^2+t*x+x^2)-6*zt*t*x*(t+x)+3*t^2*x^2)/t^2,
d2t->-(t*(t+x-5)+3*x)/(2*t),
d3t->x-zt*(t+x)/t
}/.zt->1/4*(-Sqrt[(9-t)*(1-t)]*z+t+3);


gNR[i_,t_,z_,x_] = ((
1/2*z^i/Sqrt[a*z^2-b*z+c]*Log[(d-e*z-k*Sqrt[a*z^2-b*z+c])/(d-e*z+k*Sqrt[a*z^2-b*z+c])]
)/.scoeffs);


gtNR[i_,t_,z_,x_] = ((
-1/2*z^i/Sqrt[-at*z^2-bt*z+ct]*Log[(dt-et*z-k*Sqrt[-at*z^2-bt*z+ct])/(dt-et*z+k*Sqrt[-at*z^2-bt*z+ct])]
)/.scoeffs);


jNR[i_,t_,z_,x_] = ((
(1-x/t)*z^i*((c3*d1+c1*d3)*(c2^2+8*c1*c3)-6*c1*c2*c3*d2)/(2*c1*c3*(c2^2-4*c1*c3)^2)
+z^i*((c2^2+2*c1*c3)*d2-3*c2*(c1*d3+c3*d1))/(c2^2-4*c1*c3)^2*1/Sqrt[a*z^2-b*z+c]*Log[(d-e*z-k*Sqrt[a*z^2-b*z+c])/(d-e*z+k*Sqrt[a*z^2-b*z+c])]
)/.scoeffs);


jtNR[i_,t_,z_,x_] = ((
-(1-x/t)*z^i*(2*c1*d3t-c2*d2t)/(c1*(c2^2-4*c1*c3))
-z^i*(2*c3*d2t-c2*d3t)/(c2^2-4*c1*c3)*1/Sqrt[a*z^2-b*z+c]*Log[(d-e*z-k*Sqrt[a*z^2-b*z+c])/(d-e*z+k*Sqrt[a*z^2-b*z+c])]
)/.scoeffs);

End[];
EndPackage[];
